// fDisk.c

/**
* fDisk.c / fDisk.h
*   - fdisk 명령에 필요한 함수 구현
*   - fat32 규격에 맞는 부트레코드 내용 등을 설정하여 device에 쓰기를 수행
*/   
   
   
#include "file.h"
#include "input.h"
#include "fDisk.h"

void		_____setHeaderFat32(char* drv, int fin )
{
	FAT_BPB 	*fatHeader;	

	U64 		sizeDevice ;
	U32		tempVar;
	
	fatHeader = (FAT_BPB*)malloc(sizeof(FAT_BPB)); 
	
	// SIZE of Volume 
	
	sizeDevice = GetDiskSize(fin); 
	printf("sizeDevice =%d\n",sizeDevice); 
	
	fatHeader->JmpBoot[0] = 0xEB ; 
	fatHeader->JmpBoot[1] = 0x3C ;  
	fatHeader->JmpBoot[2] = 0x90 ; 
	
	fatHeader->OEMName[0] = 'M'  ; 
	fatHeader->OEMName[1] = 'Y'  ; 
	fatHeader->OEMName[2] = 'F'  ; 
	fatHeader->OEMName[3] = 'A'  ; 
	fatHeader->OEMName[4] = 'T'  ; 
	fatHeader->OEMName[5] = '3'  ; 
	fatHeader->OEMName[6] = '2'  ; 
	fatHeader->OEMName[7] = '!'  ; 	
			
	fatHeader->BytsPerSec = 0x0200 ;  //512 
	
	// 1 ~ 256  :: 8   257 ~ 512  : 16    513 ~~~ 32  
	fatHeader->SecPerClus = 0x08   ; //********************************
	
	fatHeader->RsvdSecCnt = 0x0020 ; 
	//SecPerClus * Bytes Per Sector === Cluster Size ;;
	fatHeader->NumFATs  = 2 ; 
	fatHeader->RootEntCnt =  0x00 ;
	fatHeader->TotSec16 = 0 ; 
	fatHeader->Media = 0xF8 ;
	fatHeader->FATs16 = 0x00 ; //  
	fatHeader->SecPerTrk = 0x003F ; // not used
	fatHeader->NumHeads  = 0x00FF ;  // not used 
	fatHeader->HiddSec   = 0x20  ;  // not used 
	
	
	// (sizeDevice - sizeDevice % (SecPerClus * BytsPerSec)) / SecPerClus ...
	//tempVar = (sizeDevice-(sizeDevice%(fatHeader->SecPerClus*fatHeader->BytsPerSec))) /fatHeader->SecPerClus ;
	tempVar =  sizeDevice /fatHeader->BytsPerSec ;
	printf("totsec32=%d\n",tempVar);
	fatHeader->TotSec32  =  tempVar ;
	
	//FAT size 32 = (((sizeDevice / 512 ) / fatHeader->SecPerClus) * 4) / 512;
	fatHeader->W.Fat32.FATs32  =  fatHeader->TotSec32 / fatHeader->SecPerClus * 4 / 512 ; 
 	fatHeader->W.Fat32.ExtFlags  =  0x00 ; 
	fatHeader->W.Fat32.FSVersion  = 0x00 ; 

	fatHeader->W.Fat32.RootDirClus = 0x0002 ; 
	fatHeader->W.Fat32.FSInfo = 0x0001 ; 
	fatHeader->W.Fat32.BootRecBackSec = 0x0006 ; 
	
	fatHeader->W.Fat32.Reserved[0] = 0x00;fatHeader->W.Fat32.Reserved[1] = 0x00;   //
	fatHeader->W.Fat32.Reserved[2] = 0x00;fatHeader->W.Fat32.Reserved[3] = 0x00;  // Un Used for Future.
	fatHeader->W.Fat32.Reserved[4] = 0x00;fatHeader->W.Fat32.Reserved[5] = 0x00;  // Un Used for Future.
	fatHeader->W.Fat32.Reserved[6] = 0x00;fatHeader->W.Fat32.Reserved[7] = 0x00;  // Un Used for Future.
	fatHeader->W.Fat32.Reserved[8] = 0x00;fatHeader->W.Fat32.Reserved[9] = 0x00;  // Un Used for Future.
	fatHeader->W.Fat32.Reserved[10] = 0x00;fatHeader->W.Fat32.Reserved[11] = 0x00;
	
	fatHeader->W.Fat32.DrivNum   =   0x01 ; 
	fatHeader->W.Fat32.Reserved1 = 0x00; 
	fatHeader->W.Fat32.BootSig   = 0x29 ; 
	
	fatHeader->W.Fat32.VolumeID = tempVar; 
	
	
	fatHeader->W.Fat32.VolumeLabel[0] = 'K'; fatHeader->W.Fat32.VolumeLabel[1] = 'O';
	fatHeader->W.Fat32.VolumeLabel[2] = 'R'; fatHeader->W.Fat32.VolumeLabel[3] = 'E';
	fatHeader->W.Fat32.VolumeLabel[4] = 'A'; fatHeader->W.Fat32.VolumeLabel[5] = 'L';
	fatHeader->W.Fat32.VolumeLabel[6] = 'O'; fatHeader->W.Fat32.VolumeLabel[7] = 'V';
	fatHeader->W.Fat32.VolumeLabel[8] = 'E'; fatHeader->W.Fat32.VolumeLabel[9] = '!';
	fatHeader->W.Fat32.VolumeLabel[10] = ' '; 
	fatHeader->W.Fat32.FSType[0] = 0x46; fatHeader->W.Fat32.FSType[1] = 0x41;
	fatHeader->W.Fat32.FSType[2] = 0x54; fatHeader->W.Fat32.FSType[3] = 0x33;
	fatHeader->W.Fat32.FSType[4] = 0x32; fatHeader->W.Fat32.FSType[5] = ' ';
	fatHeader->W.Fat32.FSType[6] = ' '; fatHeader->W.Fat32.FSType[7] = ' ';
	
	fatHeader->Signature = 0xAA55 ;	
	_____SetDataToDisk(fin , 0, fatHeader , 512) ; //위의 내용으로 헤더 바꿈.
	_____SetDataToDisk(fin , 5, fatHeader , 512) ; //위의 내용으로 헤더 바꿈.
	//HexDump(fatHeader, 512);

	
	// Boot record initialize
	
	int rootDirJump =  fatHeader->RsvdSecCnt + (fatHeader->NumFATs * fatHeader->W.Fat32.FATs32);
	printf("rootDirJump=%d(%x), RsvdSecCnt=%d, NFATs=%d, FATs32=%d, SecPerClus=%d\n",
	       rootDirJump, rootDirJump, fatHeader->RsvdSecCnt,
	 	fatHeader->NumFATs, fatHeader->W.Fat32.FATs32, fatHeader->SecPerClus);
   	int secPerClus = fatHeader->SecPerClus;
	int fatJump= fatHeader->RsvdSecCnt ; 
	int nextFat =  fatHeader->W.Fat32.FATs32 ; 
	memset(fatHeader, 0 , sizeof(FAT_BPB));
	
	_____setFSInfo((char*)fatHeader);
	_____SetDataToDisk(fin , 1, fatHeader , 512) ; //위의 내용으로 헤더 바꿈.	
	_____SetDataToDisk(fin , 6, fatHeader , 512) ; //위의 내용으로 헤더 바꿈.	
	//HexDump(fatHeader, 512);


	memset(fatHeader, 0 , sizeof(FAT_BPB));
	
	int i = 0 ; 
	for(i = 0 ; i < nextFat*2 ; i++)
		_____SetDataToDisk(fin , fatJump +i  , fatHeader , 512 ) ; //위의 내용으로 헤더 바꿈.
	
	
	
	// FAT initialize
	
	memset(fatHeader, 0 , sizeof(FAT_BPB));
	for(i = 0;  i < fatHeader->W.Fat32.FATs32;  i++){ // FAT 영역 초기화
		_____SetDataToDisk(fin, fatJump+i, fatHeader, 512);
		_____SetDataToDisk(fin, fatJump+nextFat+i, fatHeader, 512);
	}
	____setFatInit( (char*)fatHeader);
	_____SetDataToDisk(fin , fatJump, fatHeader , 512) ; //위의 내용으로 헤더 바꿈.	
	//_____SetDataToDisk(fin , fatJump + nextFat , fatHeader , 512) ; // 두번째 FAT는 우리의 것 ;; label table
	
	
	
	// root directory initialize
	
	memset(fatHeader, 0 , sizeof(FAT_BPB));
	for(i = 0; i < secPerClus; i++){
		_____SetDataToDisk(fin , rootDirJump+i  , fatHeader , 512) ; //위의 내용으로 헤더 바꿈.
	}
	//GetDataFromDisk(fin, rootDirJump-1,fatHeader ,512);
	//HexDump(fatHeader,512) ;
	free(fatHeader); 
	
}

void		____setFatInit( char *string)
{
	string[0] = 0xF8 ; 	string[1] = 0xFF ; 	string[2] = 0xFF ; 	string[3] = 0x0F ;
	string[4] = 0xFF ; 	string[5] = 0xFF ; 	string[6] = 0xFF; 	string[7] = 0x0F ;
	string[8] = 0xF8 ; 	string[9] = 0xFF ; 	string[10] = 0xFF; 	string[11] = 0x0F ;
}


void		_____setFSInfo(char*  string)
{
	string[0] = 0x52 ; string[1] = 0x52 ; string[2] = 0x61 ; string[3] = 0x41 ;  
	string[484] = 0x72 ; string[485] = 0x72 ; string[486] = 0x41 ; string[487] = 0x61;
	string[488] = 0xFF ; string[489] = 0xFF ; string[490] = 0xFF ; string[491] = 0xFF;
	//*** nextFree cluster    Using this/// **//
	string[492] = 0xFF ; string[493] = 0xFF ; string[494] = 0xFF ; string[495] = 0xFF;
	string[510] = 0x55 ; string[511] = 0xAA ;   
}




U8             setRandomKey()
{       
        U8 key ; 
        time_t the_time;
        struct tm *tm_ptr;
        time(&the_time);
        tm_ptr = gmtime(&the_time);
                        
        key^= (U8)tm_ptr->tm_hour ; 
        key^= (U8)tm_ptr->tm_min ; 
        key^= (U8)tm_ptr->tm_sec ; 
        
        //printf("%d, %d, %d\n", tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec);
        //printf("RandomKey =%d/E\n",key);

        return  key ; 
}


void		_____setProtect(int fin ,char *drive)
{
	U8* buf ; 
	buf = (U8*)malloc(sizeof(U8)*512); 
	GetDataFromDisk(fin , 1, buf , 512);
        buf[32] = 'P' ;  // protect moder / /
        buf[48] = setRandomKey(); 
        
        HexDump(buf,512);
        _____SetDataToDisk(fin, 1, buf, 512);
        free(buf);
}

void            _____setWear(int  fin,char *drive)
{
         U8* buf ; 
         buf = (U8*)malloc(sizeof(U8)*512); 
        GetDataFromDisk(fin , 1, buf , 512);
        buf[33] = 'W' ;  
        _____SetDataToDisk(fin, 1, buf, 512);
        free(buf);
}

void            _____setAccess(int  fin,char *drive)
{
         U8* buf ; 
         buf = (U8*)malloc(sizeof(U8)*512); 
         GetDataFromDisk(fin , 1, buf , 512);
         buf[34] = 'A' ; 
         _____SetDataToDisk(fin, 1, buf, 512);
         free(buf);
}

void            _____setJournal(int  fin,char *drive)
{
        U8* buf ; 
        buf = (U8*)malloc(sizeof(U8)*512); 
        GetDataFromDisk(fin , 1, buf , 512);
        buf[35] = 'J' ;  
        _____SetDataToDisk(fin, 1, buf, 512);
        free(buf);
}


