// file.c//

/**
* file.c / file.h
*   - 파일 시스템에 접근하는데 사용되는 함수들 구현
*   - ls, cd, mkdir, mv, cp 등의 명령과 이에 필요한 함수들 구현
*    % 코드 상단부에 보조 함수들, 하단부에 기능 함수들 구현
*/


#include "file.h"
#include "myIntFs.h"
#include "des.h"
#include <stdio.h>


U32 		IsOpen(int* fin , char* drv) 
{
	U32 		ret = TRUE ;
	*fin = open(drv,O_RDONLY);
	
	tcflush(*fin,TCIFLUSH);
	close(*fin);	

	*fin = open(drv, O_RDWR);

	if(*fin == -1)
	{
		perror("File Open Error!");
		ret = -1 ; 
	}

	return ret ; 
}


void		fileClose(int* fin)
{
	tcflush(*fin, TCIFLUSH);
	close(*fin);
	return;
}


void		HexDump(U8 *addr, U32 len)
{
	U8		*s = addr, *endPtr = (U8*)((U32)addr+len);
	U32		i, remainder = len%16;
 		
	printf("\n Offset        Hex Value         Ascii value\n");
 	
 	// print out 16 byte blocks.
	while (s+16<=endPtr){
 		
		if( ((long)(s-addr) % 512 ) == 0 )
			printf("\n");

 		
		printf("0x%08lx  ", (long)(s-addr));

		for (i=0; i<16; i++){
			printf("%02x ", s[i]);
		}
		printf(" ");
 		
		for (i=0; i<16; i++){
			if (s[i]>=32 && s[i]<=125)printf("%c", s[i]);
			else printf(".");
		}
		s += 16;

		printf("\n");
		
		
	}
 	
 	// Print out remainder.
	if (remainder){
 		
		printf("0x%08lx  ", (long)(s-addr));
 		
		for (i=0; i<remainder; i++){
			printf("%02x ", s[i]);
		}
		for (i=0; i<(16-remainder); i++){
			printf("   ");
		}
 
		printf(" ");
		for (i=0; i<remainder; i++){
			if (s[i]>=32 && s[i]<=125) printf("%c", s[i]);
			else	printf(".");
		}
		for (i=0; i<(16-remainder); i++){
			printf(" ");
		}
		printf("\n");
	}
	return;
}	// HexDump.




void		pushFileList(char *name) // this function is used for cdFunc !!!!
{
	ListFile *t = ListFileHead->next ; 
	ListFile *s = ListFileHead  ; 
	ListFile *insert;
	insert	= (ListFile*)malloc(sizeof(ListFile)); 			
	strcpy(insert->name,name); 
	
	while(t!= ListFileTail)
	{

		t= t->next ; 
		s= s->next ; 
	}
	
	s->next = insert ; 
	insert->next = ListFileTail ; 


}



void		delFileList()
{
	ListFile *temp = ListFileHead->next;
	ListFile * t;
	while( temp != ListFileTail )
	{
		t = temp;
		temp = temp->next ;
		free(t);
	}
	ListFileHead->next = ListFileTail;
	ListFileTail->next = ListFileTail;
	
	return;
}



int		fullPathIntoList(char *fullPath)
{
	int 		i, k=0, count = 0;
	char		pushName[150];
	
	for(i=1;  fullPath[i]!='\0'; i++){
		pushName[k++] = fullPath[i] ;
		if(fullPath[i]=='/')
		{
			pushName[k-1] = '\0' ; 
			pushFileList(pushName); 
			count++;
			k=0 ;
		}
		if(fullPath[i+1]=='\0'){
			pushName[k] = '\0';
			pushFileList(pushName);
			count++;
		}
	}
	
	return count;
}


int		getMachingFolderIndex(char *name, DirEntry *pDir)
{
	int 		i, j, y, LongEntryEn;
	char		comp[150] = {0, };
	
	for (i=0; i<16; i++)//(gVol->ClusterSize/32); i++) // gVol->ClusterSize/32bytes = DirEntriesPerCluster
	{
		switch( (U8) pDir[i].Name[0]){
			case 0x00:
				return -1;	// End of Entry
			case 0xE5:
				continue;
		}
		
		if(pDir[i].Attr == 0x0F){
			LongEntryEn = 1;	      //Long File Name Entry
			continue;
		}
		
		if(pDir[i].Attr != 0x10){
			if(LongEntryEn==1) LongEntryEn = 0;
			continue;
		}

		if(LongEntryEn == 1)
		{  	// get long file name
			comp[0] = "\0";
			print_longName((LongDirEntry*)pDir, i-1, comp, 1);
			LongEntryEn = 0;
		}
		else
		{	// get short file name
			for(y=0, j= 0; y < 11; ){
				if(pDir[i].Name[y] == 0x20 || pDir[i].Name[y] == 0x00 || pDir[i].Name[y] == 0x10) break ; 
				else
					comp[j++] = pDir[i].Name[y++];
			}
			comp[j] = '\0';
		}
		
		// we will compare here
		if(!strcmp(comp, name)){
			return i;
		}
	}
	if(i==16)
		return -1;
}


int		getMachingIndex(char *name, DirEntry *pDir)
{
	int 		i, j, y, LongEntryEn;
	char		comp[150] = {0, };
	
	for (i=0; i<16; i++)//(gVol->ClusterSize/32); i++) // gVol->ClusterSize/32bytes = DirEntriesPerCluster
	{
		switch( (U8) pDir[i].Name[0]){
			case 0x00:
				return -1;	// End of Entry
			case 0xE5:
				continue;
		}
		
		if(pDir[i].Attr == 0x0F){
			LongEntryEn = 1;	      //Long File Name Entry
			continue;
		}

		if(LongEntryEn == 1)
		{  	// get long file name
			comp[0] = "\0";
			print_longName((LongDirEntry*)pDir, i-1, comp, 1);
			LongEntryEn = 0;
		}
		else
		{	// get short file name
			for(y=0, j= 0; y < 11; ){
				if(pDir[i].Name[y] == 0x20 || pDir[i].Name[y] == 0x00 || pDir[i].Name[y] == 0x10) break ; 
				else
					comp[j++] = pDir[i].Name[y++];
			}
			comp[j] = '\0';
		}
		
		// we will compare here
		if(!strcmp(comp, name)){
			return i;
		}
	}
	if(i==16)
		return -1;
}


int		findClusByPath(PWDirectory *tempPwd, int isFullPath, int fin, VolStruct gVol)
{
	ListFile 	*temp = ListFileHead->next;
	
	U8		*buf;
	DirEntry	*pDir;
	char		comp[500];
	int 	i, j, x, y, flag, LongEntryEn ;

	buf = (U8*)malloc(sizeof(U8)*512);
	strcpy(tempPwd->path, curPwd.path);
	tempPwd->num = curPwd.num;
	
	while( temp != ListFileTail )
	{
		if(isFullPath == 1) // 1=true,  0=false
		{ 
			if(temp == ListFileHead->next)
				tempPwd->num = 2;
		}
		strcpy(comp, temp->name);
		temp = temp->next;

		for(i=0; i<gVol.SecPerClus; i++)
		{
			GetDataFromDisk(fin, gVol.RootDirSec+(gVol.SecPerClus*(tempPwd->num-2))+i, buf, 512);
			pDir = (DirEntry*)buf;
			
			flag = getMachingFolderIndex(comp, (DirEntry*)buf);

			
			if(flag != -1){
				tempPwd->num = ((U32)pDir[flag].FstClusHi<<16) | ((U32)pDir[flag].FstClusLow);
				
				if(tempPwd->num <= 0x2){
					tempPwd->num = 0x2;
					strcpy(tempPwd->path, "/");
				}
				if(!strcmp(comp,"..")){
					
					for(i=0; i<512; i++)
					{	
						if(tempPwd->path[i]=='\0') break;
					}
					
					for(i; i>0; i--)
					{	
						if(tempPwd->path[i]=='/')
						{	tempPwd->path[i]='\0';
							break ;
						}
					}
				}
				else if(!strcmp(comp, ".")){}
				else if(isFullPath == 0){
					if(!strcmp(tempPwd->path, "/"))
						sprintf(tempPwd->path, "/%s", comp);
					else
						sprintf(tempPwd->path, "%s/%s", tempPwd->path, comp);
				}
				break;
			}
			else
				return -1;
		}
	}
	
	free(buf);
	return 0;
}



int		findSameName(char *name, int fin, VolStruct gVol, U32 startClusNum)
{
	U8		*buf;
	DirEntry	*pDir;
	int	i, flag;
	
	buf = (U8*)malloc(sizeof(U8)*512);
	
	while(1)
	{
		for(i=0; i<gVol.SecPerClus; i++)
		{
			GetDataFromDisk(fin, gVol.RootDirSec+(gVol.SecPerClus*(startClusNum-2))+i, buf, 512);
			flag = getMachingIndex(name, (DirEntry*)buf);
			if(flag != -1)
			{
				free(buf);
				return flag + i*1000;
			}
		}
		//printf("RDS=%x, SPC=%x\n", gVol.RootDirSec, gVol.SecPerClus);
		startClusNum = getNextCluster(startClusNum, fin, &gVol) & 0x0fffffff;
		if(startClusNum > 0x0ffffff7) break;
	}
	
	free(buf);
	return -1;
}



int		findEmptyTable(U32 *arr, int fin, VolStruct gVol)
{
	int	i, j, flag;
	U32	comp;
	U32	buf[128];
	
	flag = 0;
	for(i=420; i<gVol.FATSize; i++)
	{
		GetDataFromDisk(fin, gVol.FATStartSec+i, buf, 512);
		for(j=0; j<128; j++)
		{
			comp = buf[j];
			comp &= 0x0fffffff;
			if(comp == 0x00000000){
				flag = 1;
				break;
			}
		}
		if(flag == 1)
			break;
	}
	if(flag == 0)
		return -1;
	
	arr[0] = i;  arr[1] = j;
	
	return 1;
}




U32		addFATtable(int fin, VolStruct gVol, int sec, int bufIndex)
{	
	//printf("FSS = %d,  sec = %d,  bufIndex = %d\n", gVol.FATStartSec, sec, bufIndex);
	U32	buf[128];
	int	i, j;
	
	GetDataFromDisk(fin, gVol.FATStartSec + sec, buf, 512);
	//HexDump(buf, 512);
	buf[bufIndex] = 0x0fffffff;

	//HexDump(buf, 512);	
	//printf("FATset: sec=%d\n", sec);
	_____SetDataToDisk(fin, gVol.FATStartSec+sec, buf, 512);
	//HexDump(buf, 512);
	
	return 1;
}




void		initDirPointing(int fin, VolStruct gVol, U32 newClus, U32 ancientClus)
{	
	DirEntry	dir[16];
	memset(dir, 0, sizeof(dir));
	int	i;
	
	for(i=0; i<2; i++){
		memset(dir[i].Name, 0x20, 11);
		dir[i].Name[0] = '.';
		dir[i].Attr = 0x10;
	}
	
	dir[1].Name[1] = '.';
	
	dir[0].FstClusHi	= (U16)((newClus & 0xffff0000) >> 16);
	dir[0].FstClusLow	= (U16)(newClus & 0x0000ffff);
	
	dir[1].FstClusHi	= (U16)((ancientClus & 0xffff0000) >> 16);
	dir[1].FstClusLow	= (U16)(ancientClus & 0x0000ffff);

	//printf("initDirPointing: newClus=%d\n", newClus);
	_____SetDataToDisk(fin, gVol.FirstDataSec + ((newClus-2)*gVol.SecPerClus), dir, 512);
	//HexDump(dir, 512);
	
	return;
}



int  	 	meLong(char* name)  //-1 is long file
{
	int 		i  = 0 ,cnt= 0;
	for(i= 0 ; name[i] != '\0' ; i++,cnt++)
	{	
		if(name[i] == '.')
			cnt = 0 ; 
	}
	if(cnt == i) { } // no extend 
	else if(cnt > 4){
		//	printf(" . txt > 4 \n"); 
			return -1 ;
	}
	
	
	i -= cnt	; 
	cnt = 0 ; 
	//printf("===========================!!%c\n",name[i]);
	
	for( ; i>=0; i--,cnt++)
	{	
		if( (name[i] >= 'a' && name[i]<= 'z') || cnt > 7)
			return -1; 
	}
}



void 		addSubDirEntry(char* name, U32 fatSec, U32 newClus, int fin, VolStruct gVol, U32 curClus, U8 attr) 
{
	U8 buf[512] = {0,} ; 
	DirEntry  addOne ; 
	DirEntry* 	dir  ; 
	int i = 0 ,j= 0 ; 
	int len = 0 ;
	char nameTemp[8] , extTemp[3]; 
	
	GetDataFromDisk(fin, gVol.FirstDataSec + ((curClus-2)*gVol.SecPerClus), buf, 512 );
	
	dir = (DirEntry*)buf ; 
	i = strlen(name); 
	memset(nameTemp , 0x20,sizeof(nameTemp)); 
	memset(extTemp , 0x20, sizeof(extTemp)); 
	
	
	for( ; i > 0 ; i--)
		if( name[i] == '.')	 break ; 
	
	if(i== 0 ) // . 없는 경우. 		
	{
   		for(i= 0 ; name[i] != '\0' ; i++)	nameTemp[i] = name[i] ;  // 폴더명!!!! 
	}
	else
	{	
		i++;
		for(j=0; name[i] != '\0' ; i++,j++)
			extTemp[j] = name[i];			
		//extTemp[j] = '\0';  	
		for(i=0; name[i] != '.' ; i++)
		{
			nameTemp[i]= name[i]; 
		}	
		//nameTemp[i] ='\0' ; 
	}

	
	GetAttribute(&addOne) ;  	
	
	for( i= 0 ; i < 16 ; i++)
	{
		switch( (U8)dir[i].Name[0] )
		{
			case 0x00 : 
				memcpy(dir[i].Name, nameTemp, 8);
				memcpy(dir[i].Exten,extTemp,3); 
				dir[i].Attr 			= attr;
				dir[i].NTRes 		= 0;
				dir[i].CrtTimeTenth 	= addOne.CrtTimeTenth;
				dir[i].CrtTime 		= addOne.CrtTime;
				dir[i].CrtDate 		= addOne.CrtDate;
				dir[i].LstAccDate 	= addOne.LstAccDate; // auto
				dir[i].FstClusHi 	= (U16)(((fatSec*128 + newClus) & 0xffff0000) >> 16);
				dir[i].WriteTime 	= addOne.WriteTime; // auto
				dir[i].WriteDate 	= addOne.WriteDate; // auto. same with WriteData
				dir[i].FstClusLow 	= (U16)((fatSec*128 + newClus) & 0x0000ffff);
				dir[i].FileSize 	= addOne.FileSize;
				i =16 ; 				
				break ; 
			case 0xE5 : 
					continue ; 
		
			default :
					continue ; 
		}
	}
	//printf("sebDirEnt: curClus=%d\n", curClus);
	_____SetDataToDisk(fin, gVol.FirstDataSec + ((curClus-2) * gVol.SecPerClus), (U8*)dir, 512);
	//HexDump(dir,512);
}


void		addLongDirEntry(char *name, U32 fatSec, U32 newClus, int fin, VolStruct gVol, U32 curClus, U8 attr)
{
	int	i, entNum;
	DirEntry	dir[16];
	
	DirEntry	addOne;
	
	for(i=0; i<gVol.SecPerClus; i++)
	{
		GetDataFromDisk(fin, gVol.FirstDataSec + ((curClus-2) * gVol.SecPerClus) + i, dir, 512);
		entNum = findDirEntryOrder(dir, 0x00);
		if(entNum != -1) break;
	}
	
	if(entNum != -1){
		GetAttribute(&addOne) ; 	
	
		memcpy(dir[entNum].Name, name, 11);
		dir[entNum].Attr 		= attr;
		dir[entNum].NTRes 		= 0;
		dir[entNum].CrtTimeTenth 	= addOne.CrtTimeTenth;
		dir[entNum].CrtTime 		= addOne.CrtTime;
		dir[entNum].CrtDate 		= addOne.CrtDate;
		dir[entNum].LstAccDate 		= addOne.LstAccDate; // auto
		dir[entNum].FstClusHi 		= (U16)(((fatSec*128 + newClus) & 0xffff0000) >> 16);
		dir[entNum].WriteTime 		= addOne.WriteTime; // auto
		dir[entNum].WriteDate 		= addOne.WriteDate; // auto. same with WriteData
		dir[entNum].FstClusLow 		= (U16)((fatSec*128 + newClus) & 0x0000ffff);
		dir[entNum].FileSize 		= addOne.FileSize;

		_____SetDataToDisk(fin, gVol.FirstDataSec + ((curClus-2) * gVol.SecPerClus), (U8*)dir, 512);
		//HexDump(dir, 512);

	}
	else
		printf("클러스터 가득 참\n");
}


int		findDirEntryOrder(LongDirEntry* LongDir, U32 order)
{
	int 	i;
	for(i=0; i<16; i++)
	{
		if(LongDir[i].Order == order)
			return i;
	}
	
	return -1;
}

void		addLongNameEntries(char *name, int fin, VolStruct gVol, U32 curClus)
{
	LongDirEntry	LongDir[16];	

	char		ext[4];
	int		len,  i, j, k, entNum;
	int		q, m, indx;

	len = strlen(name);
	q = len / 13;
	m = len % 13;
	if(m != 0)
		q++;
	
	//printf("name=%s/e, len=%d\n", name, len);
	//printf("len=%d, q=%d, m=%d, name=%s\n", len, q, m, name);
	for(i=0; i<q; i++){
		for(j=0; j<gVol.SecPerClus; j++)
		{
			GetDataFromDisk(fin, gVol.FirstDataSec + ((curClus-2) * gVol.SecPerClus) + j, LongDir, 512);
			entNum = findDirEntryOrder(LongDir, 0x00);
			if(entNum != -1) break;
		}
		if(entNum != -1)
		{
			LongDir[entNum].Attr = 0x0f;
			LongDir[entNum].Order = q - i;
			if(i==0) LongDir[entNum].Order |= 0x40;
			memset(LongDir[entNum].Name1, 0, 10);
			memset(LongDir[entNum].Name2, 0, 12);
			memset(LongDir[entNum].Name3, 0, 4);
			indx = (q - i - 1) * 13;

			for(k = indx;  k< indx+13;  k++)
			{
				if(m!=0 && i==0)
				{
					if(k - indx == m){
						if( ( k - indx ) < 5 )  LongDir[entNum].Name1[(k-indx)*2] = '\0';
						else if( ( k - indx ) >=5  &&  ( k - indx ) < 11 ) 	LongDir[entNum].Name2[(k-indx-5)*2] = '\0';
						else  LongDir[entNum].Name3[(k-indx-11)*2] = '\0';
					}
					else if(k - indx > m){
						if( ( k - indx ) < 5 )  LongDir[entNum].Name1[(k-indx)*2] = 0xff;
						else if( ( k - indx ) >=5  &&  ( k - indx ) < 11 ) 	LongDir[entNum].Name2[(k-indx-5)*2] = 0xff;
						else  LongDir[entNum].Name3[(k-indx-11)*2] = 0xff;
					}
					else{
						if( ( k - indx ) < 5 )  LongDir[entNum].Name1[(k-indx)*2] = name[k];
						else if( ( k - indx ) >=5  &&  ( k - indx ) < 11 ) 	LongDir[entNum].Name2[(k-indx-5)*2] = name[k];
						else  LongDir[entNum].Name3[(k-indx-11)*2] = name[k];
					}
				}
				else{
					if( ( k - indx ) < 5 )  LongDir[entNum].Name1[(k-indx)*2] = name[k];
					else if( ( k - indx ) >=5  &&  ( k - indx ) < 11 )  LongDir[entNum].Name2[(k-indx-5)*2] = name[k];
					else  LongDir[entNum].Name3[(k-indx-11)*2] = name[k];
				}
			}
			//printf("this is Long!!!\n");
			//HexDump(LongDir,512); 
			//printf("LEntries: curClus=%d\n", curClus);
			_____SetDataToDisk(fin, gVol.FirstDataSec + ((curClus-2) * gVol.SecPerClus) + j, (U8*)LongDir, 512);
			
		}
	}
	
	return;
}


void  	GetAttribute(DirEntry* addone) 
{
	//struct tm *tm_ptr;
    	//tm_ptr = gmtime(&org_time);

	// tm_ptr->tm_year+1900,
	// tm_ptr->tm_mon+1,
	// tm_ptr->tm_mday,
	// tm_ptr->tm_hour,
	// tm_ptr->tm_min,
	// tm_ptr->tm_sec);
	addone->Attr = 0x10 ; 
	addone->NTRes = 0x00 ; 
	addone->CrtTimeTenth= 0x00 ;
	addone->CrtTime = 0x00 ; 
	addone->LstAccDate= 0x00 ; 
	addone->FstClusHi = 0x00 ;
	addone->WriteTime = 0x00 ; 
	addone->WriteDate = 0x00 ; 
	addone->FstClusLow = 0x00 ; 
	addone->FileSize = 0x00 ; 
} 



U32		getNextCluster(U32 curCluster, int fin, VolStruct *gVol)
{
	U32	FATtable[128];
	U32	ThisFATSecNum, ThisFATEntOffset, tmpFATNum;

	tmpFATNum = curCluster / 128;  // one secter (512bytes) -> there are 128 FAT sells
	ThisFATSecNum = gVol->FATStartSec + tmpFATNum;
	ThisFATEntOffset = curCluster - (tmpFATNum * 128);

	GetDataFromDisk(fin, ThisFATSecNum, FATtable, 512);
	
	return FATtable[ThisFATEntOffset];
}


U32		clusConv2Sec(U32 clusterNum, VolStruct *gVol)
{
	return ((clusterNum - 2) * gVol->SecPerClus) + gVol->FirstDataSec ;
}
		

U32 		show_dir(DirEntry* pDir, VolStruct *gVol) 
{	
	U32 i, y, period, LongEntryEn=0;		
 	
 	// 한 클러스터에 대해 검색
	for (i=0; i<16; i++)//(gVol->ClusterSize/32); i++) // gVol->ClusterSize/32bytes = DirEntriesPerCluster
	{	
		switch( (U8) pDir[i].Name[0]){
			case 0x00:	return 1;	// End of Entry
			case 0xE5:	continue;
		}
 		
		if(pDir[i].Attr == 0x0F){
			LongEntryEn = 1;	      //Long File Name Entry
			continue;
		}
 		
		//printf("---------Entry Number %d ------------ \n", i);
  
		if(pDir[i].Attr == 0x10) printf("(D) ");
		else	                      printf("(F) ");
 
		if(LongEntryEn == 1)
		{  	// print long file name
			print_longName((LongDirEntry*)pDir, i-1, "N", 0);
			LongEntryEn = 0;
		}
		else
		{	//printf short file name
			period = 0;
			for(y=0; y < 11; y++){
				if(pDir[i].Attr!=0x10 && period ==0 && !strncmp(&(pDir[i].Name[y]), " ", 1)){
					period = 1;
					printf(".");
				}
				else if(!strncmp(&(pDir[i].Name[y]), " ", 1)) {}
				else
					printf("%c",pDir[i].Name[y]);
			}
			period = 0; 
		}
		printf("\n");
		//printf("File Size     : %d \n",pDir[i].FileSize);
		//printf("Start Cluster : %d \n",(pDir[i].FstClusLow | pDir[i].FstClusHi << 16) );
	}
	return 1;
}


char		*print_longName(LongDirEntry* pLongDir, U32 EntryNum, char *buf, int use)
{
	char		filename[800];
	U32		nameOffset = 0;
	int 		z, q;
	
	do{
		memcpy(&filename[nameOffset], pLongDir[EntryNum].Name1, 10);
		nameOffset += 10;
		memcpy(&filename[nameOffset], pLongDir[EntryNum].Name2, 12);
		nameOffset += 12;
		memcpy(&filename[nameOffset], pLongDir[EntryNum].Name3, 4);
		nameOffset += 4;
	}while( EntryNum > 0 && (pLongDir[EntryNum--].Order & 0x40) == 0);
	
	filename[nameOffset] = 0x00;
	
	if(use==1){ // use bit is setted
		for(z=0, q=0 ; z < nameOffset; z+=2){
			if(filename[z]==0x00) break;
			buf[q++] = filename[z];
		}
		buf[q] = 0x00;
		return buf;
	}
	else{ // just print out
		for(z=0 ; z < nameOffset ; z+=2){
			if(filename[z]==0x00) break;
			printf("%c",  filename[z]);
		}
		return 0;
	}
}

 
 int		_____SetPointerBegin(int fin)
{
	int	size = 1 ;
	size = lseek(fin,0,SEEK_SET);		
	return size  ; 

}
 
 
U64		GetDiskSize(int fin )
{
	U64 size = _____SetPointerEnd(fin);
	_____SetPointerBegin(fin);	
	return size	; 
}	


U64		_____SetPointerEnd(int fin)
{
	U64		size = 0 ;

	size = lseek(fin,0,SEEK_END);
	return size ;

}


void	EndProgram(int g_fin)
{
	printf("End Of Program\n");

	close(g_fin); 
}


U32		GetDataFromDisk(int fin, U32 blocks, U8* buf, int size)
{
	U32 ret = FALSE;
	U64		offset = blocks  ;
	
	offset *= size ;
	memset(buf,0,sizeof(buf));
	
	lseek(fin,offset,SEEK_SET);
	
	ret = read(fin,buf,size);
	
	if(ret)
		ret = TRUE ; 
	else
		ret = FALSE ; 

	return	ret ; 
}




U32		IsFileSystem(int fin,U8* buf)
{
	int		fatSize= 0  ; 
	int		i ; 
	

	memset(buf,0,sizeof(buf));
	read(fin,buf,SIZE);

	if(buf[510] == 0x55 && buf[511] == 0xaa) // FAT 
	{	
		i =22 ; 
		for( ; i < 24 ; i++)
		{
			fatSize += buf[i] ; 
		}

		if( fatSize == 0 && buf[17] == 0 )
			return FAT32  ; 
		else
			return FAT16 ;	
	}

	return  NOTYPE ; 

}


void		MyMemCpy(U8* target,U8* input , int size)
{
	int		i=0;

	
	for(; i < size; i++)
		target[i] = input[i]; 
}



void		PrintVolume(VolStruct *gVol)
{
	printf("\n\n[[[[[[[ Volume Information ]]]]]]] \n");
	printf("RootEntCnt          =%d\n",gVol->RootEntCnt);
	printf("volBegin            =%d\n",gVol->VolBeginSec);
	printf("Total Sector      = %d sectors \n", gVol->TotalSec);
	printf("FAT size           = %d sectors \n", gVol->FATSize);
	printf("Root Dir Sector   = %d          \n", gVol->RootDirSec);
	printf("Root Dir Sector Count = %d   \n", gVol->RootDirSecCnt);
	printf("First Data Sector    = %d      \n", gVol->FirstDataSec);
	printf("Data Sector Count = %d Sectors\n", gVol->DataSecSize);
	printf("Total Cluster      = %d         \n", gVol->TotalClusCnt);
	printf("Size of Cluster    = %d         \n", gVol->ClusterSize);
	printf("FAR start sector   = %d         \n",gVol->FATStartSec);
}


U32		GetBpbInfo(FAT_BPB *BPB, VolStruct *pVol , int typefileSystem)
{
	
	//FAT 가 아니라면 return 0 ; 
	if(BPB->Signature != 0xAA55)  
		return FALSE;
	
	if(typefileSystem == FAT16 )
	{
		if( BPB->TotSec16 != 0 ) 
			pVol->TotalSec = BPB->TotSec16;	
		else			  
			pVol->TotalSec = BPB->TotSec32;	
		// Get FAT Size	
		pVol->FATSize = BPB->FATs16;
		// Get FAT Start Sector
		pVol->FATStartSec = pVol->VolBeginSec + BPB->RsvdSecCnt;
		// Get Root Dir Entry Count
		pVol->RootEntCnt = BPB->RootEntCnt;
		// Get Root Dir Sector
		pVol->RootDirSec = pVol->VolBeginSec + BPB->RsvdSecCnt + (BPB->NumFATs * BPB->FATs16);
		// Get Root Dir Sector Count;
		pVol->RootDirSecCnt = (( BPB->RootEntCnt * 32) + (BPB->BytsPerSec - 1)) / BPB->BytsPerSec;
		// GET FAT Start Sector
		pVol->FirstDataSec = pVol->VolBeginSec + BPB->RsvdSecCnt + (BPB->NumFATs * pVol->FATSize) + pVol->RootDirSecCnt;
		// Get Size of Data Area	
		pVol->DataSecSize = pVol->TotalSec - (BPB->RsvdSecCnt + (BPB->NumFATs * pVol->FATSize) + pVol->RootDirSecCnt);
		// Get Total Cluster Count;	
		pVol->TotalClusCnt = pVol->DataSecSize / BPB->SecPerClus;
		// Get Size of Cluster
		pVol->ClusterSize = BPB->SecPerClus * BPB->BytsPerSec;
		// Get Sector Per Cluster
		pVol->SecPerClus = BPB->SecPerClus;
	}
	else if(typefileSystem == FAT32)
	{
		pVol->TotalSec = BPB->TotSec32;	
		
		pVol->FATSize= BPB->W.Fat32.FATs32;
		
		pVol->RootEntCnt = 0 ;// FAT32 루트디렉토리 entcnt = 0 이다.
		//pVol->RootEntCnt = BPB->W.Fat32.RootDirClus;// FAT32 루트디렉토리 entcnt = 0 이다.
		pVol->FATStartSec =	pVol->VolBeginSec + BPB->RsvdSecCnt; 
		//루트디렉토리 섹터수는 항상 0 임 
		
		pVol->RootDirSec = pVol->VolBeginSec + BPB->RsvdSecCnt +
				(BPB->NumFATs * BPB->W.Fat32.FATs32);
		
		pVol->RootDirSecCnt = 0;
		pVol->FirstDataSec = pVol->VolBeginSec 
				+ BPB->RsvdSecCnt + (BPB->NumFATs * pVol->FATSize) + pVol->RootDirSecCnt;
				
		pVol->DataSecSize = pVol->TotalSec - (BPB->RsvdSecCnt + (BPB->NumFATs * pVol->FATSize) + pVol->RootDirSecCnt);
		// Get Total Cluster Count;	
		pVol->TotalClusCnt = pVol->DataSecSize / BPB->SecPerClus;	
		// Get Size of Cluster
		pVol->ClusterSize = BPB->SecPerClus * BPB->BytsPerSec;
		// Get Sector Per Cluster
		pVol->SecPerClus = BPB->SecPerClus;

	}

	return TRUE ;
}


void		_____SetZero(int fin,U32 start, U32 end )
{
	int i = 0 , size = end%CLU_SIZE ; 
	//char 	buf[DEL_SIZE] ={0,}; //클러스터단위.
	int		num = end / (DEL_SIZE) ; 
	char    *remain ; 
	char 	*buf  ; 
	lseek(fin,start,SEEK_SET);
	printf("num =%d =%d\n",num,end);
	sleep(1);	
	if(num!=0)
	{
		buf = (char*)malloc(sizeof(char) * DEL_SIZE );
	}
	for(;i < num ; i++)
	{
		printf("i=%d\n",i);
		write(fin,buf,DEL_SIZE);	
	}
	if(size)
	{
		remain = (char*)malloc(sizeof(char) * size);
		
		//for(i=0 ; i < size ; i++)
		//	remain[i] = 0x00;
		printf("sizeof(remain))=%d\n",i);
		write(fin,remain,i);
		free(remain);
	}
	
	if(num!=0) free(buf);
}


void		_____SetSpace(U8* string,U32 start, U32 end)
{
	// end 까지 채워줌 end가 2바이트이면 2바이트째까지 채워짐.
	// 0 - 511 이면  // 512 byte가 다채워짐.
	U32		i = start , size = end ;
	for( ; i <= size ; i++)
	{
		string[i] = 0x20 ; //space(0x20) 삽입.
	}
}




U32		_____SetDataToDisk(int fin, U32  blocks, U8* buf, int size)
{
	U32 ret = FALSE;
	int		offset = blocks ; //-1 ; 
	offset *=size ; 

	lseek(fin,offset,SEEK_SET);
	write(fin,buf,size);

	if(ret)
		ret = TRUE ;
	else
	{	
		ret = FALSE; 
	}
	return	ret ; 
}


U32		_____SetBackUpInfoToDisk(int dfin, char *fileName)
{
	FILE *fin; 
	char	buf[SIZE+1] ={0,}; 

	fin = fopen(fileName,"r+b");
	if(fin == FALSE)
		return	FALSE; 
	fread(buf,1,sizeof(buf),fin) ; // sizeof 랑 strlen !! 
	fclose(fin); 

	_____SetDataToDisk(dfin,1,(U8*)buf,SIZE); 

	return	TRUE ; 
}


U32		_____SetHeaderFat16(U8 drv, U8 size)
{

	U32  bytestoread = size, numread;
	char	buf[SIZE] ={0,};
	char    cur_drv[100]={0,} ; 

	int  g_hDevice;
//	sprintf(cur_drv,"\\\\.\\PhysicalDrive%d",(U32)drv);
// 	g_hDevice=CreateFile(cur_drv, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
//		NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
	
	/////////////////////////////////////////////////////
	////         FAT16 해더 값을 조사해서 넣어주는 함수
	////////////////////////////////////////////////////
	
//	CloseHandle(g_hDevice); 
	return		TRUE ; 
}



void 		diskChecking(char *disk)
{
	int		fin = 0 ; 
	char 		*input = NULL ; 
	char 		*output = NULL ; 
	U8		head[SIZE] = {0,}; 
	U8		*buf  ; 
	buf = (U8*)malloc(sizeof(U8)*512);
	
	IsOpen(&fin, disk);
	
	switch(IsFileSystem(fin,buf)){
		// NOTYPE,  FAT16,  FAT32,  EXT2,  MYFAT
		case NOTYPE :
			printf("File System =  NOTYPE\n");
			break;
		case FAT16 :
			printf("File System =  FAT16\n");
			break;
		case FAT32 :
			printf("File System =  FAT32\n");
			break;
		case EXT2 :
			printf("File System =  EXT2\n");
			break;
		case MYFAT :
			printf("File System =  MYFAT\n");
			break;
		default :
			printf("File System checking made an unknown error.. \n");
			break;
	}
	
	HexDump(buf,512);
	
	VolStruct 	gVol;
	
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;
	gVol.Drive = 0x02;
	gVol.VolBeginSec = 0x00;
	
	//GetDataFromDisk(int fin, U32 blocks, U8* buf, int size);
	//// U32 blocks : index to start reading
	
	//GetBpbInfo(FAT_BPB *BPB, VolStruct *pVol , int typefileSystem);
	
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	GetDataFromDisk(fin, gVol.RootDirSec, buf, secBytes);
	
	HexDump(buf,512);
	printf("ClusterSize = %d, RootDirSec = %d, BytesPerSec = %d, SecPerClus = %d\n",
	       gVol.ClusterSize, gVol.RootDirSec, secBytes, gVol.SecPerClus);
	
	GetDataFromDisk(fin, gVol.FATStartSec, buf, secBytes);
	HexDump(buf,512);
//	GetDataFromDisk(fin, gVol.FATStartSec+1, buf, secBytes);
//	HexDump(buf,512);
//	GetDataFromDisk(fin, gVol.FATStartSec+2, buf, secBytes);
//	HexDump(buf,512);
//	GetDataFromDisk(fin, gVol.FATStartSec+3, buf, secBytes);
//	HexDump(buf,512);
// 	GetDataFromDisk(fin, gVol.FATStartSec+4, buf, secBytes);
// 	HexDump(buf,512);
	printf("FATStartSec = %d\n", gVol.FATStartSec);
	
	int indexTemp = 0;
	int clusInput = 0;
	char charTemp;
	while(1){
		charTemp = mygetch();
		
		if(!strncmp(&charTemp, ":", 1)){
			scanf(": %d", &clusInput);
			indexTemp = clusInput;
			GetDataFromDisk(fin, gVol.RootDirSec + indexTemp, buf, secBytes);
			HexDump(buf,512);
			printf("%x", indexTemp);
		}
		else if(!strncmp(&charTemp, "b", 1))
			indexTemp--;
		else if(!strncmp(&charTemp, "c", 1))
			indexTemp += gVol.SecPerClus;
		else if(!strncmp(&charTemp, "C", 1))
			indexTemp -= gVol.SecPerClus;
		else if(!strncmp(&charTemp, "q", 1) || !strncmp(&charTemp, "Q", 1))
			break;
		else
			indexTemp++;
		
		if(!strncmp(&charTemp, "F", 1)){
			GetDataFromDisk(fin, gVol.FATStartSec, buf, secBytes);
			HexDump(buf,512);
			indexTemp--;
		}
		else if(!strncmp(&charTemp, "f", 1)){
			GetDataFromDisk(fin, gVol.FATStartSec + gVol.FATSize, buf, secBytes);
			HexDump(buf, 512);
			indexTemp--;
		}
		else{
			GetDataFromDisk(fin, gVol.RootDirSec + indexTemp, buf, secBytes);
			HexDump(buf,512);
			printf("%x", indexTemp);
		}
	}
	
	free(buf);
	fileClose(&fin);
	return;
}


void 	 changeLong2short(char *name,  char *target) 
{
	//char *temp = (char*)malloc(sizeof(char)*11)	;
	memset(target,0x20,11);
	
	char temp[11] ; 
	int i = 0 ,k=0, flag=0; 
	char extTemp[3];
	

	for(i = 0  ; name[i] != '\0' ; i++) {
		if( name[i] >= 'a' && name[i] <= 'z')
		{
			name[i] -=32 ; 
		}
	}
	
	//printf("fir:%s\n",name);
	for( ; i >0; i--)
		if(name[i] == '.')
			break ; 
	
	if(i!=0)
	{
		i++ ; 
		for( ; name[i] !='\0' ; i++){ extTemp[k++] = name[i] ; flag=1;}
	}
	else {
		printf("??\n");
		memset(target,0x20,11);
	}

	//printf("sec:%s\n",name);
	for(i = 0  ; name[i] != '\0' ; i++)
	{
		
		if(((name[i] >= '0' && name[i] <= '9')|| (name[i] >= 'A' && name[i] <= 'Z')||   name[i] == '$' 
		|| name[i] == '%' || name[i] == '_' || name[i] == '@'  
		|| name[i] == '!' || name[i] == '@' || name[i] == '{' ||  name[i] == '}' 
		|| name[i] == '~'  || name[i] == '^'  || name[i] == '&'  || name[i] == '#'|| name[i]=='.' )) ; 
		else		
			name[i] = '_' ; 
		
	}
	name[i] = '\0' ; 
	printf("thr:%s\n",name);
	
	for(i= 0,k=0 ; i<6 ; i++)
	{
		if(name[i] == '.' )break ; 	 ; 
		target[k++] = name[i];
				
	}
	target[k] = '~' ; target[k+1] = '1' ;
			
	
	if(flag ==1){
		target[8] = extTemp[0] ; target[9] = extTemp[1] ; target[10] = extTemp[2] ; 	}
		
}





////////////////////////////////////////////////////////////////////////
// Operation Functions
//
// - fdisk, ls, cd, mkdir, rm, cp, mv ...
// - 명령어 기능 구현 부분
//


////// ls function :::::  ls  or  ls  "dirname"
// 옵션이 없는 경우, 현재 작업 중인 디렉토리의 파일 목록을 화면에 출력
// 옵션이 있는 경우, "dirname"과 같은 경로의 파일 목록을 화면에 출력
//
void 		lsFunc() /////////////////////////////////  <   ls   >
{
	U8*		buf;
	U32		fatSell[128];
	PWDirectory	tempPwd;
	
	buf = (U8*)malloc(sizeof(U8)*512);
	
	VolStruct gVol;
	int	i;
	
	//gVol.Drive 		 = 0x2;
	gVol.VolBeginSec = 0x0;
	
	GetDataFromDisk(Me->gfin, 0, buf, 512);
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;
	
	tempPwd.num = curPwd.num;
	printf("curPwd = %d\n", curPwd.num);
	
	while(1)
	{
		for(i=0; i<gVol.SecPerClus; i++){
			GetDataFromDisk(Me->gfin, gVol.RootDirSec+(gVol.SecPerClus*(tempPwd.num-2))+i, buf, secBytes);
			show_dir((DirEntry*)buf, &gVol);
		}
	
		tempPwd.num = getNextCluster(tempPwd.num, Me->gfin, &gVol) & 0x0fffffff;
		if(tempPwd.num > 0x0ffffff7) break;
	}
	
  	free(buf);
	return;
}


////// cd function :::::  cd  "dirname"
// "dirname"과 같은 경로의 디렉토리로 현재 작업중인 디렉토리를 변경
//
void		cdFunc(char *dest) /////////////////////////////////  <   cd   >
{
	U8		*buf;
	DirEntry	*pDir;
	U32		fatSell;
	U32 		i, j, y, x, period, flag=0, LongEntryEn=0;
	U8		name[512];
	char		comp[500];
	char 		pushName[512] ; 
	PWDirectory	tempPwd;
	
	buf = (U8*)malloc(sizeof(U8)*512);
	
	VolStruct gVol;
	int 	k = 0 ;
	int	isFullPath;

	gVol.VolBeginSec = 0x0;
	
	GetDataFromDisk(Me->gfin, 0, buf, 512);
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;
	
	if(dest[0]=='\0'){
		printf("Usage : ... \n");
		return;
	}
	else if(dest[0]=='/' && dest[1]=='\0'){
		strcpy(curPwd.path, "/");
		curPwd.num = 2;
		return;
	}
	else if(dest[0]!='/'){
		isFullPath = 0;
		pushFileList(dest);
	}
	else{
		isFullPath = 1;
		fullPathIntoList(dest);
	}
	
	
	if(findClusByPath(&tempPwd, isFullPath, Me->gfin, gVol) == -1){
		printf("Cannot find the path\n");
	}
	else{
		if(isFullPath==0)
			strcpy(curPwd.path, tempPwd.path);
		else
			strcpy(curPwd.path, dest);
		curPwd.num = tempPwd.num;
	}
	
	delFileList();
	
	free(buf);
	return;
}



void  printFileList()
{
	 ListFile* t= ListFileHead->next ; 
	 while(t!= ListFileTail)
	 {
		  printf("t===>name%s\n",t->name);
		  t = t->next ; 
	 }
	 
}


////// make a directory function :::::  mkdir  "dirname"
// "dirname"의 위치에 디렉토리 정보를 추가하고,
// 새로운 클러스터를 하나 할당하여 dirname 위치에 대한 디렉토리 페이지 생성
//
void		mkdirFunc(char * name) /////////////////////////////////  <   mkdir   >
{
	U8		*buf;
	U32		arr[2];
	int		sec, i, count;
	VolStruct 	gVol;
	DirEntry 	addOne;
	PWDirectory   tempPwd ;
	char		target[512];
	int 	fullPath = 1; 
	int 	clusNum =  0 ; 

	gVol.VolBeginSec = 0x0;
	buf = (U8*)malloc(sizeof(U8)*512);
	
	GetDataFromDisk(Me->gfin, 0, buf, 512);
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;
	
	
	if(strlen(name)==0 || (strlen(name)==1 && name[0]=='/')) printf("option error!!\n");
	else if(name[0] !='/'){
		strcpy(tempPwd.path, curPwd.path);
		tempPwd.num = curPwd.num;
		strcpy(target, name);
	}
	else{
		count = 0;
		for(i=1; name[i]!='\0'; i++)
			if(name[i]=='/') count++;
		if(count==0){
			strcpy(tempPwd.path, "/");
			tempPwd.num = 2;
			strcpy(target, &name[1]);
		}
		else{
			fullPathIntoList(name);
			ListFile *t;  
		 	ListFile *s =ListFileHead;
			while(s->next != ListFileTail){	
				t = s; 
				s= s->next ; 
			}
			strcpy(target, s);
			t->next = ListFileTail;
			
			clusNum = findClusByPath(&tempPwd, fullPath, Me->gfin,gVol) ;
			if(clusNum == -1){
				printf("Error path\n");
				free(buf);
				return;
			}
		}
	}

	
	if(findSameName(name, Me->gfin, gVol, tempPwd.num) != -1) printf("Error Same Name\n"); 
	else{
		if(findEmptyTable(arr, Me->gfin, gVol) == -1)
		{ 
			printf("Disk full!!\n"); //Plz// not used 
			free(buf);
			return ; 
		}
		else{
			addFATtable(Me->gfin,  gVol, arr[0], arr[1]);
			//arr[0] = secter value ;  <---
			//arr[1] = cluster value ; <---
			// this is for setting some value into the moon ; 
			printf("target=%s\n", target);
	  		 if (meLong(target) == -1){ 
				addLongNameEntries(target, Me->gfin, gVol, tempPwd.num);			
				char target2[20] ;
				changeLong2short(target,target2) ;//  malloc으로 바꾸어서 넣어야 안전하다.
				addLongDirEntry(target2, arr[0], arr[1], Me->gfin, gVol, tempPwd.num, 0x10);
			}
			else
				addSubDirEntry(target ,arr[0], arr[1] , Me->gfin, gVol, tempPwd.num, 0x10) ;
			cleanCluster(Me->gfin, gVol, (((U32)arr[0]*128)+(U32)arr[1]));
			initDirPointing(Me->gfin, gVol, (((U32)arr[0]*128)+(U32)arr[1]), (U32)tempPwd.num);
		}
	}
	
	delFileList();
	free(buf);
	return;
}


////// remove function :::::  rm  "target"
// "target"의 디렉토리 정보와 파일 내용 삭제
//
void		rmFunc(char *option) /////////////////////////////////  <   rm   >
{
	PWDirectory 	tempPwd;
	DirEntry		*pDir;
	VolStruct 		gVol;
	int	isFullPath;
	int	result;
	U8	*fatSell;
	U8	*buf;
	
	int 		i, isRecur=0, count=0;
	char		dest[200];
	char		target[200];
	
	gVol.VolBeginSec = 0x0;
	buf = (U8*)malloc(sizeof(U8)*512);
	
	GetDataFromDisk(Me->gfin, 0, buf, 512);
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;
	
	printf("RDS=%d, FDS=%d, fin=%d\n", gVol.RootDirSec, gVol.FirstDataSec, Me->gfin);

	
	strcpy(target, option);
	
	for(i=0; target[i]!='\0'; i++)
	{
		if(target[i]==' '){
			target[i] = '\0';
			if(!strcmp(target, "r")){
				isRecur = 1;
				strcpy(target, &option[i+1]);
				break;
			}
			else{
				isRecur = -1;
				break;
			}
		}
	}
	

	if(isRecur==-1){
		printf("rm option error!!\n");
		free(buf); return;
	}
	
	
	if(strlen(target)==0 || (strlen(target)==1 && target[0]=='/')){
		printf("option error!!\n");
		free(buf); return;
	}
	else if(target[0] !='/'){
		strcpy(tempPwd.path, curPwd.path);
		tempPwd.num = curPwd.num;
		strcpy(dest, target);
	}
	else{
		count = 0;
		for(i=1; target[i]!='\0'; i++)
			if(target[i]=='/') count++;
		if(count==0){
			strcpy(tempPwd.path, "/");
			tempPwd.num = 2;
			strcpy(dest, &target[1]);
		}
		else{
			fullPathIntoList(target);
			ListFile *t;  
		 	ListFile *s =ListFileHead;
			while(s->next != ListFileTail){	
				t = s; 
				s= s->next ; 
			}
			strcpy(dest, s);
			t->next = ListFileTail;
			
			if(findClusByPath(&tempPwd, 1, Me->gfin,gVol) == -1){
				printf("Error path\n");
				free(buf);
				return;
			}
		}
	}
	printf("dest=%s, target=%s, tempPwd.num=%d\n", dest, target, tempPwd.num);
	
	
	if((result = findSameName(dest, Me->gfin, gVol, tempPwd.num)) == -1){
		printf("No Same Name\n");
		free(buf); return;
	}
	
	
	GetDataFromDisk(Me->gfin, gVol.FirstDataSec + ((tempPwd.num-2)*gVol.SecPerClus) + (result/1000), buf, 512);
	pDir = (DirEntry*)buf;
	
	if(isRecur==0){
		if(pDir[result%1000].Attr == 0x10){
			printf("Directory cannot remove!!\n");
			free(buf); return;
		}
	}
	pDir[result%1000].Name[0] = 0xe5;
	
	_____SetDataToDisk(Me->gfin, gVol.FirstDataSec + ((tempPwd.num-2)*gVol.SecPerClus) + (result/1000), buf, 512);

	// FAT sell remove
	GetDataFromDisk(Me->gfin, gVol.FATStartSec + (tempPwd.num/128), buf, 512);
	fatSell = (U32*)buf;
	fatSell[tempPwd.num] = 0x0;
	_____SetDataToDisk(Me->gfin, gVol.FATStartSec + (tempPwd.num/128), buf, 512);
	

	delFileList();
	
	free(buf);
	return;
}


int		findTargetNClusByPath(char *target, char *path, PWDirectory *tempPwd, int fin, VolStruct gVol)
{
	int	count, i;
	
	// target -> path / dest -> target
	if(strlen(path)==0 || (strlen(path)==1 && path[0]=='/')){
		printf("option error!!\n");
		return -1;
	}
	else if(path[0] !='/'){
		strcpy(tempPwd->path, curPwd.path);
		tempPwd->num = curPwd.num;
		strcpy(target, path);
	}
	else{
		count = 0;
		for(i=1; path[i]!='\0'; i++)
			if(path[i]=='/') count++;
		if(count==0){
			strcpy(tempPwd->path, "/");
			tempPwd->num = 2;
			strcpy(target, &path[1]);
		}
		else{
			fullPathIntoList(path);
			ListFile *t;  
		 	ListFile *s =ListFileHead;
			while(s->next != ListFileTail){	
				t = s; 
				s= s->next ; 
			}
			strcpy(target, s);
			t->next = ListFileTail;
			
			if(findClusByPath(tempPwd, 1, fin, gVol) == -1){
				printf("Error path\n");
				delFileList();
				return -1;
			}
		}
	}
	delFileList();
	return 0;
}


////// move function :::::  mv  "source"  "dest"
// "source" 위치의 디렉토리 정보를 "dest" 위치에 복사하여 "dest"가 해당 파일이 저장된 클러스터를 링크하도록 한 후
// "source"의 디렉토리 정보 삭제
//
void		mvFunc(char *option) /////////////////////////////////  <   mv   >
{
	PWDirectory 	targetPwd, destPwd;
	DirEntry		*pDir;
	VolStruct 		gVol;
	
	int		isFullPath;
	int		result;
	U8		*fatSell;
	U8		*buf;
	U32		targetClus;
	
	int 		i, count=0;
	char		targetPath[400];
	char		target[200];
	char		destPath[400];
	char		dest[200];
	U8		attr;
	
	gVol.VolBeginSec = 0x0;
	buf = (U8*)malloc(sizeof(U8)*512);
	
	GetDataFromDisk(Me->gfin, 0, buf, 512);
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;

	// mv  "target"  "dest"
	
	strcpy(targetPath, option);
	
	for(i=0; targetPath[i]!='\0'; i++)
	{
		if(targetPath[i]==' '){
			targetPath[i] = '\0';
			strcpy(destPath, &option[i+1]);
			break;
		}
	}
	if(strlen(option)==strlen(targetPath)){
		printf("option Error : there is no destination\n");
		free(buf); return;
	}
	
	findTargetNClusByPath(target, targetPath, &targetPwd, Me->gfin, gVol);
	findTargetNClusByPath(dest, destPath, &destPwd, Me->gfin, gVol);
	
	printf("dest=%s/e, target=%s/e, destPath=%s\e, destPwd.num=%d, targetPwd.num=%d\n", dest, target, destPath, destPwd.num, targetPwd.num);
	
	
	// dest checking ...
	if(findSameName(dest, Me->gfin, gVol, destPwd.num) != -1){
		printf("There's a Same Name with Dest!!\n");
		free(buf); return;
	}
	
	
	// target erasing ...
	if((result = findSameName(target, Me->gfin, gVol, targetPwd.num)) == -1){
		printf("No Same Name with Target\n");
		free(buf); return;
	}
	
	GetDataFromDisk(Me->gfin, gVol.FirstDataSec + ((targetPwd.num-2)*gVol.SecPerClus) + (result/1000), buf, 512);
	pDir = (DirEntry*)buf;
	
	pDir[result%1000].Name[0] = 0xe5;
	targetClus = (U32)pDir[result%1000].FstClusLow;
	targetClus = targetClus | ((U32)(pDir[result%1000].FstClusHi) << 16);
	attr = pDir[result%1000].Attr;
	
	printf("targetClus = %x\n", targetClus);
	printf("target erasing\n");
	_____SetDataToDisk(Me->gfin, gVol.FirstDataSec + ((targetPwd.num-2)*gVol.SecPerClus) + (result/1000), buf, 512);

	
	// dest writing ...
	if (meLong(destPath) == -1){ 
		addLongNameEntries(destPath, Me->gfin, gVol, destPwd.num);			
		char shortDest[20] ;
		changeLong2short(dest, shortDest) ;//  malloc으로 바꾸어서 넣어야 안전하다.
		addLongDirEntry(shortDest, (U16)(targetClus/128), (U16)(targetClus%128), Me->gfin, gVol, destPwd.num, attr);
		printf("This is LLLLLLongggggggFile !1!!\n");
	}
	else
		addSubDirEntry(dest, targetClus/128, targetClus%128, Me->gfin, gVol, destPwd.num, attr) ;
	
	
	
	delFileList();
	
	free(buf);
	return;
}


////// copy frome home OS to portable device :::::  h2p  "source"  "dest"
// home OS의 "source" 위치에 있는 파일을 portable device의 "dest" 위치에 복사
//
void		home2portableFunc(char *option) ////////////////////// < cp from home to portable >
{
	PWDirectory  targetPwd, destPwd;
	DirEntry  *pDir;
	VolStruct   gVol;
	
	int  	isFullPath;
	int  	result;
	U8  	*fatSell;
	U8  	*buf;
	U8		*inputStream;
	U8		*outStream;
	U32  	targetClus;
	
	int   i, count=0;
	char  targetPath[400]={0,};
	char  target[200]={0,};
	char  destPath[400]={0,};
	char  dest[200]={0,};

	U8  	attr;
	U32	arr[2];
		
	int   isRecur = 0 ;
	
	FILE 	*fhome;
	
	
	gVol.VolBeginSec = 0x0;
	buf = (U8*)malloc(sizeof(U8)*512);
	
	inputStream = (U8*)malloc(sizeof(U8)*512);
	memset(inputStream, 0, sizeof(inputStream));
	
	outStream = (U8*)malloc(sizeof(U8)*512);
	memset(outStream, 0, sizeof(outStream));
	
	
	GetDataFromDisk(Me->gfin, 0, buf, 512);
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;

	// h2p  "target"  "dest"
	printf("h2p option tokening is started.\n");
	
	strcpy(targetPath, option);
	for(i=0; targetPath[i]!='\0'; i++)
	{
		if(targetPath[i]==' '){
			targetPath[i] = '\0';
			strcpy(destPath, &option[i+1]);
			break;
		}
	}
	if(strlen(option)==strlen(targetPath)){
		printf("option Error : there is no destination\n");
		free(buf); return;
	}

	
	printf("source : %s\ndest : %s\n", targetPath, destPath);
	
	findTargetNClusByPath(dest, destPath, &destPwd, Me->gfin, gVol);

	// dest checking ...
	if(findSameName(dest, Me->gfin, gVol, destPwd.num) != -1){
		printf("There's a Same Name with Dest!!\n");
		free(buf); return;
	}
	else{
		rmFunc(destPath);
		printf("file will be overwrited..\n");
	}
	
	// dest writing ...
	if(findEmptyTable(arr, Me->gfin, gVol) == -1){ 
		printf("Disk full!!\n");
		free(buf);	return ;
	}
	
	addFATtable(Me->gfin,  gVol, arr[0], arr[1]);
	if (meLong(dest) == -1){ 
		addLongNameEntries(dest, Me->gfin, gVol, destPwd.num);			
		char shortTarget[20] ;
		changeLong2short(dest, shortTarget) ;// malloc으로 바꾸어서 넣어야 안전하다.
		addLongDirEntry(shortTarget, arr[0], arr[1], Me->gfin, gVol, destPwd.num, 0x20);
	}
	else
		addSubDirEntry(dest, arr[0], arr[1], Me->gfin, gVol, destPwd.num, 0x20) ;
	
	cleanCluster(Me->gfin, gVol, ((U32)arr[0]*128 + (U32)arr[1]));
	
	
	fhome = fopen(targetPath, "r");
	fread(inputStream+4, 508, sizeof(char), fhome);
	
	printf("fread is finished\nIS ::: %s\n", inputStream);
	
	if(Me->isProtect == 1){
		//EnProtect(char *string,int strlen,int BIT_NUM,int BYTE_NUM)
		//EnProtect(inputStream, strlen(inputStream), Me->key, -1);
		
		//encrypt_des( unsigned short *input, int input_len, unsigned char *output, unsigned short key);
		encrypt_des((unsigned short *)inputStream, sizeof(inputStream), outStream, Me->key);
		HexDump(outStream, 512);
	}
	_____SetDataToDisk(Me->gfin, gVol.FirstDataSec + (((U32)arr[0]*128) + (U32)arr[1] -2)*gVol.SecPerClus, outStream, 512);
	
	
	free(inputStream);
	free(outStream);
	
	fclose(fhome);
	delFileList();

	free(buf);
	return;
}

////// copy from portable device to home OS :::::  p2h  "source"  "dest"
// portable device에 있는 "source"의 내용을 home OS의 "dest" 위치에 복사
//
void		portable2homeFunc(char *option) ////////////////////// < cp from portable to home >
{
	PWDirectory 	targetPwd;
	DirEntry		*pDir;
	VolStruct 		gVol;
	
	char		aCh;
	int		result;
	U8			*fatSell;
	U8			*buf;
	char		*inputStream;
	U32		targetClus;
	U32		arr[2];
	
	int 		i, j, count=0;
	char		targetPath[512];
	char		target[512];
	char		destPath[512];
	char		dest[512];
	U8			attr;
	
	FILE		*fhome;
	
	gVol.VolBeginSec = 0x0;
	buf = (U8*)malloc(sizeof(U8)*512);
	
	inputStream = (U8*)malloc(sizeof(U8)*512);
	memset(inputStream, 0, sizeof(inputStream));
	
	GetDataFromDisk(Me->gfin, 0, buf, 512);
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;
	
	
	GetDataFromDisk(Me->gfin, gVol.FATStartSec, buf, 512);
	
	strcpy(targetPath, option);
	for(i=0; targetPath[i]!='\0'; i++)
	{
		if(targetPath[i]==' '){
			targetPath[i] = '\0';
			strcpy(destPath, &option[i+1]);
			break;
		}
	}
	if(strlen(option)==strlen(targetPath)){
		printf("option Error : there is no destination\n");
		free(buf); return;
	}
	
	printf("source : %s\ndest : %s\n", targetPath, destPath);
	fhome = fopen(destPath, "w+");
	
	findTargetNClusByPath(target, targetPath, &targetPwd, Me->gfin, gVol);
	
	///// target finding ...
	// target reading !!
	if((result = findSameName(target, Me->gfin, gVol, targetPwd.num)) != -1){
		GetDataFromDisk(Me->gfin, gVol.FirstDataSec + ((targetPwd.num-2)*gVol.SecPerClus) + (result/1000), buf, 512);
		pDir = (DirEntry*)buf;
		targetClus = (U32)(pDir[result%1000].FstClusHi) << 16;
		targetClus = targetClus | (U32)(pDir[result%1000].FstClusLow);
	
		for(i=0; i<gVol.SecPerClus; i++){
			GetDataFromDisk(Me->gfin, gVol.FirstDataSec + ((targetClus-2)*gVol.SecPerClus) + i, buf, 512);
			if(Me->isProtect == 1){
				//DeProtect(char *string,int strlen,int BIT_NUM,int BYTE_NUM)
				//DeProtect(buf, strlen(buf), Me->key, -1);
				
				HexDump(buf, 512);
				//decrypt_des(unsigned short *input, int input_len, unsigned char *output, unsigned short key);
				decrypt_des((unsigned short*)buf, sizeof(buf), inputStream, Me->key);
				
				HexDump(inputStream, 512);
			}
			for(j=4; j<512; j++){
				if(inputStream[j]==0x0a && inputStream[j+1]==0x00){
					i = 8;
					break;
				}
				printf("%c", inputStream[j]);
				fprintf(fhome, "%c", inputStream[j]);
			}
			if(inputStream[j]==0x0a && inputStream[j+1]==0x00)
				break;
		}
		fprintf(fhome, "\n\0");
	}
	else{
		puts("there is no target file.");
	}
	
	free(inputStream);
	
	fclose(fhome);
	free(buf);
	return;
}


////// copy function :::::  cp  "source"  "dest"
// "source"의 파일과 파일 정보를 "dest"에 저장
// "dest"의 디렉토리 정보 클러스터에 파일이 추가되며,
// 비어있는 클러스터를 할당해 연결하고 soure의 내용을 복사
//
void  	cpFunc(char *option) /////////////////////////////////  <   cp   >
{
	PWDirectory  targetPwd, destPwd;
	DirEntry  *pDir;
	VolStruct   gVol;
	
	int  isFullPath;
	int  result;
	U8  *fatSell;
	U8  *buf;
	U32  targetClus;
	
	int   i, count=0;
	char  targetPath[400]={0,};
	char  target[200]={0,};
	char  destPath[400]={0,};
	char  dest[200]={0,};
	
	char  dest1[200] ={0,};
	char  dest2[200] ={0,};
	U8  attr;
		
	int   isRecur = 0 ;
	
	FILE *fp;
	
	gVol.VolBeginSec = 0x0;
	buf = (U8*)malloc(sizeof(U8)*512);
	
	GetDataFromDisk(Me->gfin, 0, buf, 512);
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;

	// cp  "target"  "dest"
	
	strcpy(targetPath, option);
	int k = 0 ;
	int    isRmode = FALSE ; 
	for(i=0; targetPath[i]!='\0'; i++)
	{
		if(targetPath[i]==' '){
			targetPath[i] = '\0';
			strcpy(destPath, &option[i+1]);
			if(!strcmp(targetPath, 'r'))
			{
				strcpy(dest2,destPath) ; 
				isRmode = TRUE ; 
				for(k = 0 ; destPath[k]!= '\0' ; k++){
					if(destPath[k]== ' ')
					{
						destPath[k] = '\0' ; // first
						strcpy(dest1,&dest2[k+1]) ; //second ; 
						break ; 
					}    
				}
												
			}
			break;
		}
	}
	if(strlen(option)==strlen(targetPath)){
		printf("option Error : there is no destination\n");
		free(buf); return;
	}
	
	if(isRmode == TRUE)
	{       
		strcpy(targetPath,destPath); 
		strcpy(destPath,dest1); 
	}

	
	//this part is Not implemmented  . .
	if(isRecur==TRUE){
		printf("cp option error!!\n");
		free(buf); return;
	}
	
	findTargetNClusByPath(target, targetPath, &targetPwd, Me->gfin, gVol);
	findTargetNClusByPath(dest, destPath, &destPwd, Me->gfin, gVol);


	// dest checking ...
	if(findSameName(dest, Me->gfin, gVol, destPwd.num) != -1){
		printf("There's a Same Name with Dest!!\n");
		free(buf); return;
	}
	// target erasing ...
	if((result = findSameName(target, Me->gfin, gVol, targetPwd.num)) == -1){
		printf("No Same Name with Target\n");
		free(buf); return;
	}

	
	////////////////////////////////////////////////////////////////////
	//             get all Name in dest !!
	//             one name per copy.
	///////////////////////////////////////////////////////////////////
	GetDataFromDisk(Me->gfin, gVol.FirstDataSec + ((targetPwd.num-2)*gVol.SecPerClus) + (result/1000), buf, 512);
	pDir = (DirEntry*)buf;

	targetClus = (U32)pDir[result%1000].FstClusLow;
	targetClus = targetClus | ((U32)(pDir[result%1000].FstClusHi) << 16);
	attr = pDir[result%1000].Attr;


	// dest writing ...
	if (meLong(destPath) == -1){ 
		addLongNameEntries(destPath, Me->gfin, gVol, destPwd.num);   
		char shortDest[20] ;
		changeLong2short(dest, shortDest) ;//  malloc으로 바꾸어서 넣어야 안전하다.

		addLongDirEntry(shortDest, (U16)(targetClus/128), (U16)(targetClus%128), Me->gfin, gVol, destPwd.num, attr);
	}
	else
		addSubDirEntry(dest, targetClus/128, targetClus%128, Me->gfin, gVol, destPwd.num, attr) ;


	delFileList();

	free(buf);
	return;
}




void		cleanCluster(int fin, VolStruct gVol, U32 curClus){
	int	i;
	U8	buf[512] = {0, };
	for(i=0; i<gVol.SecPerClus; i++){
		_____SetDataToDisk(fin, gVol.FirstDataSec + ((curClus-2)*gVol.SecPerClus) + i, buf, 512);
	}
}


////// cat function :::::  cat  "file name"
// "file name"과 동일한 이름의 파일이 존재하지 않으면 사용자로부터 문자열 입력을 받아서 저장,
// 파일이 존재하면 해당 파일의 내용을 화면에 출력.
//
void		catFunc(char *option) /////////////////////////////////  <   cat   >
{
	PWDirectory 	targetPwd;
	DirEntry		*pDir;
	VolStruct 		gVol;
	
	char		aCh;
	int		result;
	U8		*fatSell;
	U8		*buf;
	char		*inputStream;
	U32		targetClus;
	U32		arr[2];
	
	int 		i, j, count=0;
	char		targetPath[512];
	char		target[512];
	U8		attr;
	
	gVol.VolBeginSec = 0x0;
	buf = (U8*)malloc(sizeof(U8)*512);
	inputStream = (U8*)malloc(sizeof(U8)*512);
	
	GetDataFromDisk(Me->gfin, 0, buf, 512);
	GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
	secBytes = ((FAT_BPB*)buf)->BytsPerSec;
	
	
	GetDataFromDisk(Me->gfin, gVol.FATStartSec, buf, 512);
	
	if(strlen(option)==0 || strlen(option)==512){
		printf("option error..\n");
		free(buf); return;
	}
	
	strcpy(targetPath, option);
	
	findTargetNClusByPath(target, targetPath, &targetPwd, Me->gfin, gVol);
	
	///// target finding ...
	
	// target writing !!
	if((result = findSameName(target, Me->gfin, gVol, targetPwd.num)) == -1){
		printf("New File Name!!  >> input mode start .....\n> ");
		
		if(findEmptyTable(arr, Me->gfin, gVol) == -1){ 
			printf("Disk full!!\n");
			free(buf);	return ;
		}
		else{
			addFATtable(Me->gfin,  gVol, arr[0], arr[1]);
	  		 if (meLong(target) == -1){ 
				addLongNameEntries(target, Me->gfin, gVol, targetPwd.num);			
				char shortTarget[20] ;
				changeLong2short(target, shortTarget) ;// malloc으로 바꾸어서 넣어야 안전하다.
				addLongDirEntry(shortTarget, arr[0], arr[1], Me->gfin, gVol, targetPwd.num, 0x20);
			}
			else
				addSubDirEntry(target ,arr[0], arr[1] , Me->gfin, gVol, targetPwd.num, 0x20) ;
			cleanCluster(Me->gfin, gVol, ((U32)arr[0]*128 + (U32)arr[1]));
		}
		
		for(i=4; i<512; i++){
			aCh = mygetch();
			switch(aCh){
				case 0x04 ://EOT :
					inputStream[i] = 0x0a; // EOF
					goto endInput;
				case '\n' :
					inputStream[i] = aCh;
					printf("\n> ");
					break;
				default :
					inputStream[i] = aCh;
					putc(aCh, stdout);
					break;
			}
		}
		endInput :
		printf("%s\ndestClus = %d, arr[1]=%d, FDS = %d\n", inputStream, (((U32)arr[0]*128) + (U32)arr[1] ), arr[1], gVol.FirstDataSec);
		
		if(Me->isProtect == 1){
			//EnProtect(char *string,int strlen,int BIT_NUM,int BYTE_NUM)
			//EnProtect(inputStream, strlen(inputStream), Me->key, -1);
			memset(buf, 0, sizeof(buf));
			encrypt_des((unsigned short *)inputStream, sizeof(inputStream), buf, Me->key);
			HexDump(buf, 512);
		}
		_____SetDataToDisk(Me->gfin, gVol.FirstDataSec + (((U32)arr[0]*128) + (U32)arr[1] -2)*gVol.SecPerClus, buf, 512);
		
	}
	// target reading !!
	else{
		GetDataFromDisk(Me->gfin, gVol.FirstDataSec + ((targetPwd.num-2)*gVol.SecPerClus) + (result/1000), buf, 512);
		pDir = (DirEntry*)buf;
		targetClus = (U32)(pDir[result%1000].FstClusHi) << 16;
		targetClus = targetClus | (U32)(pDir[result%1000].FstClusLow);
	
		for(i=0; i<gVol.SecPerClus; i++){
			GetDataFromDisk(Me->gfin, gVol.FirstDataSec + ((targetClus-2)*gVol.SecPerClus) + i, buf, 512);
			if(Me->isProtect == 1){
				//DeProtect(char *string,int strlen,int BIT_NUM,int BYTE_NUM)
				//DeProtect(buf, strlen(buf), Me->key, -1);
				memset(inputStream, 0, sizeof(inputStream));
				decrypt_des((unsigned short*)buf, sizeof(buf), inputStream, Me->key);
				HexDump(inputStream, 512);
			}
			for(j=4; j<512; j++){
				if(inputStream[j]==0x0a && inputStream[j+1]==0x00){
					i = 8;
					break;
				}
				printf("%c", inputStream[j]);
			}
		}
	}
	
	free(inputStream);
	
	free(buf);
	return;
}


