// file.h

#ifndef 	_FILE_
#define 	_FILE_ 


#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>

#define		TEST			"/dev/sda4"
#define		DEL_SIZE    	4096		
#define		CLU_SIZE		4096 
#define		FAT_1G  		"/dev/sda6"
#define 		FIRST_USB  	"/dev/sdb"
#define 		SECOND_USB	"/dev/sdc"
#define		SIZE			512
#define		U8		unsigned char
#define		S8		char
#define		U16		unsigned short
#define		U32		unsigned int
#define		U64		unsigned long long 

enum		{ NOTYPE , FAT16 , FAT32, EXT2, MYFAT }  ; 

enum 		{ EXIT,OPEN,LS,CP,DEL,ORDER }; 

#pragma pack(1) 
typedef	struct _PART_FAT16{

	U8		driveNum; 
	U8		reserved1;
	U8		bootSignature; 
	U32		volumeId;
	U8		volumeLevel[11]; 
	U32		fileSystemType[2]; 
	U32		noValue[7];
}PART_FAT16 ; 

typedef struct _PART_FAT32{
	U32		FATs32;
	U16		ExtFlags;
	U16		FSVersion;
	U32		RootDirClus;
	U16		FSInfo;
	U16		BootRecBackSec;
	U8		Reserved[12];
	U8		DrivNum;
	U8		Reserved1;
	U8		BootSig;
	U32		VolumeID;
	U8		VolumeLabel[11];
	U8		FSType[8];
}PART_FAT32 ; 

typedef struct _FAT_BPB_struct{
	U8		JmpBoot[3];
	U8		OEMName[8];
	U16		BytsPerSec;
	U8		SecPerClus;
	U16		RsvdSecCnt;
	U8		NumFATs;
	U16		RootEntCnt;
	U16		TotSec16;  
	U8		Media;  // USB 0xF8
	U16		FATs16; // In FAT 32,must Set 0 ;  
	U16		SecPerTrk; // 63    not used in windows
	U16		NumHeads;  // 255
	U32		HiddSec;   // 32
	U32		TotSec32;
	union {
		PART_FAT16		Fat16;
		PART_FAT32		Fat32;
	}W;
	U8		BootCodeArea[420];
	U16		Signature;
}FAT_BPB;
#pragma pack()

#pragma pack(1)
typedef struct _DIR_struct{
	U8		Name[8];
	U8 		Exten[3] ; 
	U8		Attr;
	U8		NTRes;
	U8		CrtTimeTenth;
	U16		CrtTime;
	U16		CrtDate;
	U16		LstAccDate;
	U16		FstClusHi;
	U16		WriteTime;
	U16		WriteDate;
	U16		FstClusLow;
	U32		FileSize;
}DirEntry;
#pragma pack()

#pragma pack(1)
typedef struct _LONG_DIR_struct{
	U8		Order;
	U8		Name1[10];
	U8		Attr;
	U8		Type;
	U8		chksum;
	U8		Name2[12];
	U16		FstClusLo;
	U8		Name3[4];	
}LongDirEntry;
#pragma pack()

typedef struct _VOL_struct{
	U32		Drive;
	U32		VolBeginSec;
	U32		FirstDataSec;
	U32		RootDirSec;
	U32		RootEntCnt;	
	U32		RootDirSecCnt;
	U32		FATSize;
	U32		FATStartSec;
	U32		TotalClusCnt;
	U32		TotalSec;
	U32		DataSecSize;
	U32		ClusterSize;
	U32		SecPerClus;
}VolStruct;


typedef struct fat_table{
	U32		offset;
	char		filename[100];
	short		dirty;
	U32		temp;
	char		owner[50];
}FatTable;


// how can we store present working directory
// .. we need its full path & the cluster number where the directory page is at
typedef struct	_present_working_directory{
	char		path[512];
	U32		num;
}PWDirectory;

PWDirectory	curPwd;


typedef struct  _listfile{
	char 	name[512] ; 
	struct  _listfile 		*next ; 
}ListFile ; 

ListFile  *ListFileHead , *ListFileTail ; 

U32		currentCluster;
int 		secBytes;

void		GetBackUpInfo(U8 *buf,char *fileName);
U64	_____SetPointerEnd(int fin);
void	_____SetZero(int fin,U32 start, U32 end );
int	_____SetPointerBegin(int fin);
void	_____SetSpace(U8* string,U32 start, U32 end) ;
void	_____SetFat16(U8* buf,int fileSystemType);
U32	_____SetDataToDisk(int fin,U32  blocks,U8* buf,int size);
U32	_____SetBackUpInfoToDisk(int dfin, char *fileName);
U32	_____SetHeaderFat16(U8 drv, U8 size);

int		BackUpDiskInfo(U8 drv , char* fileName,int blocks) ;

U32		IsFileSystem(int fin,U8* buf);
U32 		IsOpen(int* fin , char* drv)  ;
void		fileClose(int* fin);

void 		diskChecking(char *disk);
void		EndProgram(int g_fin);
U32 		show_dir(DirEntry* pDir, VolStruct *gVol);
char		*print_longName(LongDirEntry* pLongDir, U32 EntryNum, char *buf, int use);
void		PrintVolume(VolStruct *gVol);

void		MyMemCpy(U8* target,U8* input , int size);

void		PrintSize();
void		HexDump(U8 *addr, U32 len);


U64		GetDiskSize(int fin );
U32		GetDataFromDisk(int fin,U32 blocks,U8* buf,int size);
U32		GetBpbInfo(FAT_BPB * BPB,VolStruct *pVol , int typefileSystem);

void 		lsFunc();
void		mkdirFunc(char * name) ; 
void		rmFunc(char *option);
void		mvFunc(char *option);
void		catFunc(char *option);

U32		getNextCluster(U32 curCluster, int fin, VolStruct *gVol);
U32		clusConv2Sec(U32 clusterNum, VolStruct *gVol);

int		findEmptyTable(U32 *arr, int fin, VolStruct gVol);
//U32 		addSubDirEntry(DirEntry* pDir, DirEntry addOne, VolStruct gVol, int sec);
U32		addFATtable(int fin, VolStruct gVol, int sec, int bufIndex);


void		pushFileList(char *name);
void		delFileList();

//**********
int		findClusByPath(PWDirectory *tempPwd, int isFullPath, int fin, VolStruct gVol);
int		findSameName(char *name, int fin, VolStruct gVol, U32 startClusNum);
void 		addSubDirEntry(char* name, U32 fatSec, U32 newClus ,int fin, VolStruct gVol, U32 curClus, U8 attr) ; 
void		addLongDirEntry(char *name, U32 fatSec, U32 newClus, int fin, VolStruct gVol, U32 curClus, U8 attr);
void		initDirPointing(int fin, VolStruct gVol, U32 newClus, U32 ancientClus);

void  	GetAttribute(DirEntry* addone)  ; 
int  		meLong(char* name)   ; 
void		addLongNameEntries(char *name, int fin, VolStruct gVol, U32 curClus);

int		findDirEntryOrder(LongDirEntry* LongDir, U32 order) ;

void 	 	changeLong2short(char *name,  char *target) ; 

int		getMachingFoldersIndex(char *name, DirEntry *pDir);
int		getMachingIndex(char *name, DirEntry *pDir);

int		findTargetNClusByPath(char *target, char *path, PWDirectory *tempPwd, int fin, VolStruct gVol);
void		cleanCluster(int fin, VolStruct gVol, U32 curClus);

#endif


