// input.c

/**
* input.c / input.h
*   - 쉘 프로그램이 구동된 후 받는 모든 입력을 버퍼에 담고 엔터가 들어올 때마다 파싱하여 명령어 추출
*   - 명령어와 옵션을 인식하여 해당되는 함수를 적절한 파라미터를 전달하면서 호출
*   - 파일 시스템을 읽을 필요가 없는 간단한 명령어들은 직접 구현
*/

#include "fDisk.h"
#include "file.h"
#include "input.h"
#include "myIntFs.h"


void            printUsage(int type)  
{
    switch(type)
    {
        case FDISK : 
		printf("\nFDISK Usage**"); 
		printf("\nFDISK Usage**");
		break;
	default :
		printf("\n\nshell start at start session...\n\n");
		printf("if you want to go into a device, use < open > command ... that is..\n\n");
		printf("\n** open command ... **\n	open <device name> \n\n");
		printf("have a nice shelling..\n\n");
		break; 
    }
}


void		getOption(char *option, char* input, int index)
{
	int i=0;
	for( ; input[index+1]!='\0';index++)
	{
		if(input[index+1] == '-' ) ;
		else
			option[i++] = input[index+1] ;
	}
	option[i] = '\0' ;
}


void		pushHistory(int*	hisIndex) 
{
	if(*hisIndex > 4 )
		*hisIndex = -1;
	
	(*hisIndex)++;
	
}

int		getFdiskOpt(char*  string, FDISKOPT* value , char* drive)
{
	int 	i, j=0, len;
	char	target[100];
	char	opt[150];

	if((len = strlen(string)) == 0)
		return FALSE;
		
	strcpy(opt, string);
	
	for(i=0; opt[i]!='\0'; i++)
	{
		if(opt[i]==' '){
			opt[i] = '\0';
			strcpy(target, &string[i+1]);
			break;
		}
	}
	if(strlen(opt)==len)
		strcpy(drive, opt);
	else{
		for( i = 0 ; opt[i] != '\0'  ; i++) {
			if(opt[i] == 'w')
			{ 
				value->wear = 1 ; 
			}
			else if(opt[i] == 'p')
			{
				value->protect = 1 ; 
			}
			else if(opt[i] == 'j')
			{
		
				value->journal =1 ; 
			}
			else if(opt[i] == 'a')
			{	
				value->access = 1;
			}
			else
			{
				return FALSE;
			}	
		}
		strcpy(drive, target);
	}
	return TRUE ; 
}


int		 popHistory(int* hisIndex)
{
	if(*hisIndex == 0 )
		*hisIndex = 4  ;
	return --(*hisIndex) ; 
}


int		openDevice(char *option)
{
	if(IsOpen(&(Me->gfin), option) == -1){
		printf("wrong device!! \n");
		return 1;
	}
	
	strcpy(Me->device, option);
	initProcess();
	
	return 0;
}


void		initAllHeaders()
{
	gUserHead = (USER*)malloc(sizeof(USER)); 
	gUserTail = (USER*)malloc(sizeof(USER)); 

	gUserHead->next = gUserTail ; 
	gUserTail->next = gUserTail ;//Head ;

	strcpy(gUserHead->name,"root");
	strcpy(gUserHead->key,ROOTPASSWORD);
	
	
	ListFileHead = (ListFile*)malloc(sizeof(ListFile)) ; 
	ListFileTail = (ListFile*)malloc(sizeof(ListFile)) ;  

	ListFileHead->next = ListFileTail ; 
	ListFileTail->next = ListFileTail ; 
	
	//printf("ListFileHead[%x]->next=[%x] ,ListFileTail[%x]\n",ListFileHead,ListFileHead->next,ListFileTail);
	
	Me = (MYSTATE*)malloc(sizeof(MYSTATE));

	strcpy(Me->device, "StartSession");
	
	
}


int     initProcess()
{
        U8     buf[512] , flag = 0; 
        GetDataFromDisk(Me->gfin , 1 , buf , 512) ; 
        
        if(buf[32] == 'P' ){
		Me->isProtect = 1 ; Me->key = buf[48] ;
        }
        if(buf[33] == 'W' ){
		Me->isWear = 1 ;
		flag = 1;
        }
        if(buf[34] == 'A'){
		Me->isAccess = 1 ; 
		flag = 1;
        }
        if(buf[35] == 'J'){
		Me->isJournal = 1 ;
        	flag = 1;
        }
        
        if(flag == 1){
		VolStruct gVol;
		GetDataFromDisk(Me->gfin, 0, buf, 512);
		GetBpbInfo((FAT_BPB*)buf, &gVol, FAT32);
		Me->labelTableStartSec = gVol.FATStartSec + gVol.FATSize;
	}
}


void 		start() ////////////////////////////////////// start !!!! //////////////
{
	char 	input[512]= {0,} ; 
	char		firstCmd[512] ={0,} ;
	char		option[512] ={0,} ;
	char		arrow[2][2] ;
	char  	buffer[512];
	char 	ch ; 
	int		arCount = 0 ;
	int 		i = 0 ; 
	int 		k = 0 ;
	int		clear = 0;
	int		hisIndex = 0 ;
	int		isStart = 1;
	
	int		backIndex = 0 ; 

	arrow[0][0] = 0x5b;
	arrow[0][1] = 0x43; // right arrow
	arrow[1][0] = 0x5b;
	arrow[1][1] = 0x44; // left arrow
	
	strcpy(curPwd.path, "/");
	curPwd.num = 2;
	
	printf("*****************************************************\n\n");
	printCmdInit();
	
	while(1)
	{
		ch = mygetch(); // my own get charactor function
		putc(ch,stdout); // charactor echo..
		if(ch == '\n')
		{
			input[i] = '\0';
			strcpy(stringHistory[hisIndex].command,input);
			pushHistory(&hisIndex); 
			
			for( i= 0;input[i] != '\0' ;i++)
			{
				if(input[i] ==' '){
					break ;
				}
				firstCmd[i] = input[i];
				
			}
			firstCmd[i] = '\0'; 
			getOption(option,input,i);
			
			// matching input stream with implemented commands
			if(!strcmp(firstCmd,"login"))
			{
				login();
			}
			if(!strcmp(firstCmd,"exit"))
			{
				if(isStart==1){
					politeFarewell();
					break ; 
				}
				else{
					strcpy(Me->device, "StartSession");
					close(Me->gfin);
					isStart = 1;
				}
			}
			if(!strcmp(firstCmd, "fdisk"))
			{
				FDISKOPT  	type ; 			
				char 	drive[20] ; 
				memset(&type,0,sizeof(FDISKOPT));	
				if(( getFdiskOpt(option, &type, drive)) == FALSE)
				{
					printUsage(FDISK) ; 
					printCmdInit();
					firstCmd[0] = '\0'; 
					input[0] = '\0' ; 
					option[0] = '\0' ;
					i= 0 ; 
					arCount = 0; 
					continue ; 
				}
				fdiskFunc(&type, drive) ; 
				//fdiskFunc(&type, TEST) ;  
					
			}
			if(!strcmp(firstCmd,"ls"))
			{
				if(isStart == 0) lsFunc();
				else	system("ls");
			}
			if(!strcmp(firstCmd,"clear"))
			{
				for(clear = 0; clear<80; clear++){
					putc('\n', stdout);
				}
				for(clear = 0; clear<80; clear++){
					putc(0x1b, stdout);
					putc(0x5b, stdout);
					putc(0x41, stdout);
				}
			}
			if(!strcmp(firstCmd,"rm"))
			{
				if(isStart == 0) rmFunc(option);
			}
			if(!strcmp(firstCmd,"mkdir"))
			{
				if(isStart == 0) mkdirFunc(option);
			}
			if(!strcmp(firstCmd,"h2p"))
			{
				if(isStart == 0) home2portableFunc(option);
			}
			if(!strcmp(firstCmd,"p2h"))
			{
				if(isStart == 0) portable2homeFunc(option);
			}
			if(!strcmp(firstCmd,"cp"))
			{
				if(isStart == 0) cpFunc(option);
			}
			if(!strcmp(firstCmd, "cd"))
			{
				if(isStart == 0) cdFunc(option);
				else chdir(option);
			}
			if(!strcmp(firstCmd, "mv"))
			{
				if(isStart == 0) mvFunc(option);
			}
			if(!strcmp(firstCmd, "cat"))
			{
				if(isStart == 0) catFunc(option);
			}
			if(!strcmp(firstCmd, "pwd"))
			{
				if(isStart == 0) printf(">>%s\n", curPwd.path);
				else system("pwd");
			}
			if(!strcmp(firstCmd, "echo"))
			{
				printf("%s\n", option);
			}
			if(!strcmp(firstCmd, "open"))
			{
				if(option[0]=='/'){
					isStart = openDevice(option);
					printf("%s is opened..  option :  p = %d,  w = %d,  j = %d,  a = %d\n", Me->device, Me->isProtect, Me->isWear, Me->isJournal, Me->isAccess);
					printf("Key = %d\n", Me->key);
				}
				else
					printf("똑바로 입력하세요.\n");
			}
			if(!strcmp(firstCmd, "test"))
			{
				 setRandomKey();
			}
			if(!strcmp(firstCmd, "testing"))
			{
				diskChecking(option);
			}
			if(!strcmp(firstCmd, "new"))
			{
				int 	nameCount, inputCount = 5, flag;
				char	aCh;
				char NewName[100] ;
				char NewPass[100] ;
				if(!strcmp(option, "user")){
					while(inputCount){
						flag = 0;
						printf("User Name : ");
						for(nameCount = 0; nameCount < 100; nameCount++){
							aCh = mygetch();
							if( (aCh>='A' && aCh<='Z') || (aCh>='a' && aCh<='z') || (aCh>='0' && aCh<='9')){
								printf("%c", aCh);
								NewName[nameCount] = aCh;
							}
							else if( aCh == '\n') {
								printf("\nPassword : ");
								NewName[nameCount] = 0x00;
								flag++;
								break;
							}
							else{
								printf("UserName에 특수기호 불가. 다시 입력하세요.\n");
								break;
							}
						}
						for(nameCount = 0; nameCount < 100; nameCount++){
							aCh = mygetch();
							if( (aCh>='A' && aCh<='Z') || (aCh>='a' && aCh<='z') || (aCh>='0' && aCh<='9')){
								printf("*");
								NewPass[nameCount] = aCh;
							}
							else if( aCh == '\n') {
								printf("\n");
								NewPass[nameCount] = 0x00;
								flag++;
								break;
							}
							else{
								printf("Password에 특수기호 불가. 다시 입력하세요.\n");
								break;
							}
						}
						
						if(flag==2){
							new_user(NewName, NewPass);
							break;
						}
						
						if(inputCount-- == 0){
							printf("입력 횟수 초과\n");
							break;
						}
					}
				}
					
			}
			
			firstCmd[0] = '\0'; 
			input[0] = '\0' ; 
			option[0] = '\0' ;
			i= 0 ; 
			arCount = 0;
			printCmdInit();
		}
		else if(ch == 0x1b){
			ch = mygetch();
			if(ch == 0x5b){
				ch = mygetch();
				if(ch == 0x41) // up key
				{
					// backIndex = strlen(stringHistory[hisIndex].command);
					// while(backIndex)
					// {
						// printf("\b");
						// backIndex--; 
					// }
					
					putc(stringHistory[popHistory(&hisIndex)].command[0],stdout); //up key
					printf("%s",stringHistory[popHistory(&hisIndex)].command);
					strcpy(input,stringHistory[hisIndex].command);		
					i = strlen(stringHistory[hisIndex].command);
				}
				else if(ch == 0x42) // down key
				{
					pushHistory(&hisIndex);
					putc(stringHistory[hisIndex].command[0],stdout); //up key
					printf("%s",stringHistory[hisIndex].command);
				
				}
				else if(ch == 0x43){ // right arrow
					if(arCount<=i){
						putc(0x5b,stdout);
						putc(0x43,stdout);
						i++;
						arCount++;
					}
				}
				else if(ch == 0x44){ // left arrow
					
					if(arCount>=i && arCount>0){
						putc(0x5b,stdout);
						putc(0x44,stdout);
						if(i > 0 )
						{	arCount--;
						i--;
						}
					}
				}
			}
		}
		else if( ch == 0x7f)
		{
			printf("\b");
			if(i>0){
				printf("\b \b");
				i--;
				arCount--;
			}
		}
		else if(ch == 0x09)
		{	
			printf("\b\b\b\b");
			
		}
		else
		{
			input[i++] = ch ;
			arCount++;
		}
		
		
		
	}

}


char		*myGetpass(char* key, char* target) 
{
	int result = 0 ; 
	int bit ; 
	int byte ;
	int change=0; 
	char deProtectedPass[50];
	
	change = MyAtoI(key);
	printf("key =%d\n",change);
	bit  = (change % 6 )+1 ; 
	byte = (change % 30)+1 ;
	printf("key =%d\n",change);
	
	strcpy(deProtectedPass, target);
	printf("target=%s\n", deProtectedPass);
	DeProtect(deProtectedPass, strlen(deProtectedPass), bit, byte);
	printf("target=%s\n", deProtectedPass);
	
	return deProtectedPass;
}


void		makeKeyToPass(int key,char* target) 
{
	int bitChange = (key % 6) +1 ; 
	int byteChange = (key & 30) +1 ;
	EnProtect(target,strlen(target),bitChange,byteChange);  

}
int		makePassToKey(char* pass)
{

	int	i = 0 ; 
	unsigned int plus = 0 ;
	for( i= 0 ; pass[i] != '\0' ; i++)
	{
		plus += pass[i]  ; 
	}
	return (int)plus ; 
}

void 		new_user(char *newuser, char *newpass)
{
	char buffer[200];
	char *tk;
	char	key[50] ={0,} ; 
	int 	i = 0 ;
	int    keyValue = 0 ; 		
	
	FILE *fUsers;
	char *string ;
	string = (char*)malloc(sizeof(char) * 50);
	
	if(access("./USERS.txt", F_OK)!=0){ // if it doesn't exist...
		//fclose(fUsers);
		fUsers = fopen("USERS.txt", "w"); // make it...
		fclose(fUsers); // this situation also means there was no users...
	}
	else{ // there are some users..
		fUsers = fopen("USERS.txt", "r");
		while( (fgets(buffer, 200, fUsers)) != NULL){
			tk = strtok(buffer, "/");
			if(!strcmp(newuser, tk)){
				printf("That username is already taken!\n");
				return;
			}
		}
		fclose(fUsers);
	}
	
	fUsers = fopen("USERS.txt", "a");
	
	for(; newpass[i] != '\0' ;i++)
		string[i] = newpass[i] ; 
	string[i] = '\0';
	keyValue =makePassToKey(string);
	makeKeyToPass(keyValue,string);
	
	printf("Here!!!!!!!!!!!!!!!!!!!!!!!\n");
	fprintf(fUsers, "%s/ND%s/ND%s/ND\n", newuser, string, myltoa(keyValue ,key,16)); 
	fclose(fUsers);
	
	free(string);
	
	return;
}


void 		inputNoEcho(char * target)
{
	
	int  i=0 ; 
	struct termios oldt, newt;
	char	 ch;
	char	ascii='*' ; 
	tcgetattr( STDIN_FILENO, &oldt );
	newt = oldt;
	newt.c_lflag &= ~( ICANON | ECHO );
	tcsetattr( STDIN_FILENO, TCSANOW, &newt );
	
	while( (ch = getchar()) != '\n')
	{
		target[i++]= ch ;
		putc(ascii,stdout);
	}
	target[i] = '\0'; 
	printf("\n");
	tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
}


void		login()
{
	char 	name[50] ; 
	char 	pass[50] ;
	int		cnt= 0 ; 
	int 	result = 0 ; 
		
	do{
		printf("\nUser Name:: "); 
		gets(name);
		printf("Pass Word:: ");
		inputNoEcho(pass);
		fflush(stdin);
		//printf("N:%s,PW:%s\n", name, pass);
		
		if((result = findUser(name,pass)) >=2 ){
			strcpy(Me->user, name);
			printf("User!!! %s\n", Me->user);
			break ;
		}
		else
		{
			printf("잘못입력하였습니다.\n");
			result = 0 ;
		}
		cnt++; 
		if(cnt==LOGIN_END_COUNT)
		{	printf("program end 5번 miss\n");  
		exit(0);
		}
	
	}while(cnt != LOGIN_END_COUNT ) ; 
	
}


int	findUser(char *name,char *pass) 
{
		
	USER * t = gUserHead->next ;
	int		result = 0;
	char		tempPass[50];
		
		
	if(strcmp(name,"root") == 0)
	{
		if(strcmp(pass,ROOTPASSWORD) == 0)
			result = 3 ;
			
		return result ; 
	}
		
	while( t != gUserTail)
	{	
		if((strcmp(t->name,name))== 0 )
		{	
			result = 1 ;
			printf("t->key =%s\n",t->key);
			strcpy(tempPass, myGetpass(t->key, t->pass));
			if(strcmp(tempPass, pass) == 0)
				result = 2 ; 
					
			break ; 
		}
		t= t->next ; 
	}		
	return   result  ; 
		
}


void		read_users() 
{
	FILE*	fin ; 
	char 	buffer[200] ={0,}; 
	char	name[30] ={0,} ; 
	char 	pass[40] ={0,} ;
	char	key[20] ={0,}  ;  
	int 		i = 0 , k = 0 ;
	
	
	if((fin = fopen(READFILE,"r"))== NULL) 
		printf("Error!!\n") ;
	else
	{
			
		while(fgets(buffer, 200, fin) != NULL)
		{
			if(buffer[0] =='\n')
				break;
			for(i=0 ,k=0; buffer[i] !='\n'; i++,k++)
			{
				if(buffer[i] == '/' && buffer[i+1] == 'N' && buffer[i+2] == 'D')
				{
					name[i] = '\0' ; 	
					break ; 
				}
				name[k] = buffer[i] ; 
			}
			
			i = i+ 3 ;
			for	( k = 0 ; buffer[i] != '\n' ; i++,k++)
			{	
				if(buffer[i] == '/' && buffer[i+1] == 'N' && buffer[i+2] == 'D')
				{	pass[k] = '\0' ; break ; }
				pass[k] = buffer[i]	; 
			}
			i = i +3; 
			for	( k = 0 ; buffer[i] != '\n' ; i++,k++)
			{
				if( buffer[i] == '/' && buffer[i+1] == 'N' && buffer[i+2] == 'D')
				{
					key[k]= '\0' ; 
					break ; 
				}
				key[k] = buffer[i] ; 
			}
			pushUser(name , pass ,key); 
		}
	}
	fclose(fin); 
}

void		pushUser(char* name,char* pass,char* key) 
{
	USER 	*t  = gUserHead->next ;
	USER	*l  = gUserHead ; 
	USER 	*i ;
	
	while(t !=gUserTail)
	{
		l = l->next ; 
		t = t->next ; 
	}
	i = (USER*)malloc(sizeof(USER)); 
	
	strcpy(i->name,name);
	strcpy(i->pass,pass);
	strcpy(i->key,key);
	
	
	l->next = i ; 
	i->next = t;
	

}

void		delReadUser()
{
	USER 	*t  = gUserHead->next ;
	USER 	*l  = gUserHead  ;
	
	while( t != gUserTail)
	{
		l = t ; 
		t = t->next ; 
		free(l); 
	}
	gUserHead->next = gUserTail ; 
	gUserTail->next = gUserTail ; 

}

void		printReadUser() 
{
	USER 	*t  = gUserHead->next ;
	while( t != gUserTail)
	{
		printf("Name=%s\n",t->name);
		printf("pass=%s\n",t->pass);
		printf("key=%s, size=%d\n",t->key, strlen(t->key));
		t= t->next ; 
	}

}



void		printStart()
{
	printf("\n\n#############################################################################\n");
	printf("##                                                                         ##\n");
	printf("##           IF SHELL ::: for Security J  !!                                ##\n");
	printf("##              by hizorisa team ( our security FileSystem )                ##\n");
	printf("##                                                                         ##\n");
	printf("#############################################################################\n\n");
}

void		printCmdInit()
{
	if(!strcmp(Me->user, "root"))
		printf("\n[%s@%s]:%s# ", Me->user, Me->device, curPwd.path);
	else
		printf("\n[%s@%s]:%s$ ", Me->user, Me->device, curPwd.path);
}

void		politeFarewell()
{
	printf("\nIFShell is exited... \n\nGood bye - \n\n\n");
}

char *   myltoa (long val, char *buf, unsigned radix)
{
	char *p;		
	char *firstdig;	
	char temp;	    
	unsigned digval;

	p = buf;

	if (radix == 10 && val < 0) {
		*p++ = '-';
		val = (unsigned long)(-(long)val);
	}

	firstdig = p;	   

	do {
		digval = (unsigned) (val % radix);
		val /= radix;       

	    
		if (digval > 9)
			*p++ = (char) (digval - 10 + 'a');  
		else
			*p++ = (char) (digval + '0');       
	} while (val > 0);


	*p-- = '\0';	   

	do {
		temp = *p;
		*p = *firstdig;
		*firstdig = temp; 
		--p;
		++firstdig;	 
	} while (firstdig < p); 

	return buf;
}

int MyAtoI(const char *parm_string)
{
	int count = strlen(parm_string), i = 0;
	int hexa = 0;
	for(i = 0; i < count; i++){
		if(*parm_string >= '0' && *parm_string <= '9') 
			hexa = hexa*16 + *parm_string - '0';
		else if(*parm_string >= 'A' && *parm_string <= 'F') 
			hexa = hexa*16 + *parm_string - 'A' + 10;
		else if(*parm_string >= 'a' && *parm_string <= 'f') 
			hexa = hexa*16 + *parm_string - 'a' + 10;
		
		parm_string++;
	}
	return hexa;
}


char 		mygetch( )
{
	struct termios oldt, newt;
	char	 ch;
	tcgetattr( STDIN_FILENO, &oldt );
	newt = oldt;
	newt.c_lflag &= ~( ICANON | ECHO);
	tcsetattr( STDIN_FILENO, TCSANOW, &newt );
	ch = getchar();
	tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
	return ch;
}


////// fdisk function
// fdisk function caller.
//
int			fdiskFunc(FDISKOPT*  type, char * drive) 
{
    	int fin ; 
	
  	//  IsOpen(&fin,drive); 
	printf("drive=%s/e\n", drive);
	if(IsOpen(&fin, drive) == FALSE)
	{
		printf("drive error : there is no matching dev... \n");
		return;
	}
	_____setHeaderFat32(drive , fin ) ; 
	//  FAT 데이터가 있는 영역을 가서 다 지워주고 0으로 초기화하고. 
	//  우리가 알고 있는 값으로 해더를 채워주고 / /
	

	//  옵션 플래그 설정
	if(type->wear == 1){
		printf("W\n");
	}
	if(type->protect == 1){
		_____setProtect(fin,drive);
	}
	if(type->access == 1){
		//printf("A\n");
		_____setAccess(fin,drive);
	}
	if(type->journal == 1){
		printf("J\n");
	}
	close(fin);
	
	
	return  FALSE ; 	
}



